/*
FILE:     extract.C
*/
/*
VERSION:  3.28
*/
/*
DATE:     4/15/2021
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2021 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* the input coordinate file either is PDF format or CIF */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "rcsb_math.h"
#include <unistd.h>
#include <sys/types.h>

#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"

#include "nrutil.h"
#include "util.h"
#include "match.h"
#include "sg_nsym.h"
#include "atom_mass.h"
#include "version.h"
#include "harv.h"

#include "structure.h"

#define NMISC 29
#define MAXL 512

struct tmp{
    char space_group[20];
    char a[20];
    char b[20];
    char c[20];
    char alpha[20];
    char beta[20];
    char gamma[20];
    int mol_id;
} cell;

FILE  *fcgi=NULL;
char METHOD[80];  //X-RAY or NMR

extern char INFO[1000];
extern FILE *FW_LOG;

void catstring(char *orig, const char *ss1, const char *ss2, const char *ss3);
void write_poly_from_cif(char *iFile);
void get_template_cif();

int main(int argc, char **argv)
{
    char inpfile[MAXL], xyzfile[MAXL], xyzfileo[MAXL], str[MAXL],tmp[MAXL];
    int  i, chain=1, pdb=0,cif=0;
    FILE *fp=NULL;
    
    remove("extract.log");
    FW_LOG=fopen("extract.log","w");

    if(argc == 1 || (argc ==2 && !strcmp_case(argv[1], "-h"))){
        get_local_file(tmp,"instruction-extract");
        if((fp = fopen(tmp, "r"))==NULL){ open_file_error(tmp, "extract"); }
        while (fgets(str, sizeof str, fp) != NULL) { printf("%s",str);}
        return 0;
    }else if (argc<=2){
        sprintf(INFO, "\nError: Not enough arguments (type 'extract -h' for details).\n");
        pinfo(INFO, 0);
        return 0;
    }

  //  pid=getpid();

    strcpy(METHOD,"XRAY");
    for(i=0; i<argc; i++){
        if(!strcmp_case(argv[i], "-NMR")){
            strcpy(METHOD,"NMR");
          
        }else if (!strcmp_case(argv[i], "-EM")){
            strcpy(METHOD,"EM");
        }
    }
    
    fcgi=fopen("cgi_value", "w"); /* export a file for cgi (web page)*/
    strcpy(xyzfile,"");
    for(i=0; i<argc-1; i++){
        if(!strcmp_case(argv[i], "-PDB") || !strcmp_case(argv[i], "-IPDB")||
           !strcmp_case(argv[i], "-CIF") || !strcmp_case(argv[i], "-ICIF")){
//            clean_file(argv[i+1], xyzfile);
            
            strcpy(xyzfile, argv[i+1]);
            sprintf(inpfile, "%s_tmp", argv[i+1]);
    
        }else if(!strcmp_case(argv[i], "-EXT")){
            strcpy(inpfile,argv[i+1]);
            get_mmcif(inpfile);
            exit(0); 
        }else if(!strcmp_case(argv[i], "-chain")){
            strcpy(inpfile,argv[i+1]);
            add_chain_to_pdb(inpfile);
            
        }else if(!strcmp_case(argv[i], "-cif2pdb")){
            sprintf(inpfile, "%s_TMP.pdb",argv[i+1]);
            cif2pdb(argv[i+1], inpfile);
            printf("The output = %s\n", inpfile);
            exit(0);
        
        }else if (!strcmp_case(argv[i], "-SOL")){
            strcpy(inpfile,argv[i+1]);
     //       write_matth_coeff_to_data_template(inpfile);
            printf("\nThe new file (%s) has been generated\n",inpfile);
            
        }
        
    }

    if(strlen(xyzfile)>0){ //use coordinate
            
        remove("log_script.inp");
        remove("data_template.text");
        remove("entity-poly.mmcif");

        pdb=is_pdb(xyzfile, &chain);

        if(pdb>0 && chain==0){
            fprintf(fcgi, "PDB_chain=NO\n");
            fclose(fcgi);
                
            sprintf(INFO, "\nError: The PDB file (%s) has no Chain-ID.\n",inpfile);
            pinfo(INFO, 0);
        
            printf("Each copy of molecules must be assigned a Chain-ID (using A~Z, a~z,0~9)\n");
            printf("If one of the following conditions is satisfied.\n");
            printf(" 1. only one copy of molecule in the file. \n");
            printf(" 2. several copies in the file, but each copy is separated by TER. \n");
            printf(" 3. each copy is separated by MODEL/ENDMDL.\n");
            printf("you can use the command to generate chain ID.\n");
            printf("\nextract -chain pdb_file_name\n\n");
            exit(0);
        }
        if(pdb>0){
            get_pdb_one_model(xyzfile, inpfile);
            read_pdb_head(inpfile); //get cells
            fprintf(fcgi, "a=%s:b=%s:c=%s:alpha=%s:beta=%s:gamma=%s:space_group=\"%s\"\n",
                    cell.a, cell.b, cell.c, cell.alpha, cell.beta, cell.gamma, cell.space_group); 

            align_use_coord(inpfile);
            
//            if(key==1) {
//                if (!align_use_seqres(inpfile)) align_use_coord(inpfile);
//            }
            
            delete_file(inpfile);
            fclose(fcgi); 
            get_template_cif();
            output_info();
        
        }else{
            cif=is_cif(xyzfile);

            if (cif<3){
                sprintf(INFO, "\nError: The input file (%s) is neither in mmCIF no in PDB format.(Stopped!)\n",inpfile);
                pinfo(INFO, 0);
                fclose(FW_LOG);
                exit(0);
                
            }else{
                strcpy(xyzfileo, "EXTRACT_tmp.xyz");
                clean_file(xyzfile, xyzfileo);
                write_poly_from_cif(xyzfileo);
                remove(xyzfileo);
                get_template_cif();
                output_info();
//            entity_poly_from_cif(xyzfile);
            }

        }
    }
    fclose(FW_LOG);
}

void  get_template_cif()
// get the template file from the data directory and add entity poly & cell
{
    int i, nline;
    char  **line, str[9000];
    FILE *fp1, *fw;
    
    if((fp1=fopen("entity-poly.mmcif", "r"))==NULL) {
        open_file_error("entity-poly.mmcif", "get_template_cif");
    }

    fw=fopen("data_template.cif", "w");

    get_local_file(str,"data_template.cif");   
    line=get_lines_from_file(str, &nline);
    for(i=0; i<nline; i++){ fprintf(fw, "%s",line[i]); }
    
    fprintf(fw, "\n#\n_symmetry.entry_id     UNNAMED\n");
    fprintf(fw, "_symmetry.Int_Tables_number        ?\n" );
    fprintf(fw, "_symmetry.space_group_name_H-M    '%s'\n", cell.space_group);

    fprintf(fw, "\n#\n_cell.entry_id      UNNAMED\n");
    fprintf(fw, "_cell.length_a  %s\n" , cell.a);
    fprintf(fw, "_cell.length_b  %s\n" , cell.b);
    fprintf(fw, "_cell.length_c  %s\n" , cell.c);
    fprintf(fw, "_cell.angle_alpha  %s\n" , cell.alpha);
    fprintf(fw, "_cell.angle_beta  %s\n" , cell.beta);
    fprintf(fw, "_cell.angle_gamma  %s\n#\n" , cell.gamma);

    while (fgets(str, sizeof str, fp1) != NULL) {fprintf(fw, "%s", str); }
    
    fclose(fp1);
    fclose(fw);
     
}

void entity_poly_from_cif(char *ifile)
//overwrite the cgi_value and entity-poly.mmcif (temp)
{
    char **entity_id, **seq, **seq_type, **chid;
    long i, n1=0;
    CifFile * fobjR = NULL;
    string diags, blockId;

    CifParser* cifParserR = NULL;
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(ifile, diags);
    delete(cifParserR);

    if (!diags.empty()) {
        fprintf(stderr, "Diags for file %s: %s\n",ifile,diags.c_str());
    }
    
    blockId = fobjR->GetFirstBlockName();
    entity_id=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "entity_id", &n1);
    seq=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "pdbx_seq_one_letter_code", &n1);
    if (!seq) seq=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "ndb_seq_one_letter_code", &n1);
    seq_type=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "type", &n1);
    chid=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "pdbx_strand_id", &n1);
    if (!chid) chid=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "ndb_strand_id", &n1);

    if ( !(entity_id && seq && seq_type && chid)){
        printf("Error: can not parse entity_poly from the cif file (%s), n=%ld\n", ifile, n1);
        return;
    }

    fcgi=fopen("cgi_value", "w"); /* export a file for cgi (web page)*/
    fprintf(fcgi, "a=%s:b=%s:c=%s:alpha=%s:beta=%s:gamma=%s:space_group=\"%s\"\n",
            cell.a, cell.b, cell.c, cell.alpha, cell.beta, cell.gamma, cell.space_group);

    
    for(i=0; i<n1; i++){
//        delete_return(seq[i]);
//        delete_return([i]);
        
        replace_char(seq[i], '\n', ' ');
        replace_char(chid[i], '\n', ' ');
        fprintf(fcgi,"ENTITY_ID = %s : ENTITY_TYPE=%s : ONE_LETTER_SEQUNCE=\"%s\" : CHAIN_ID=%s : TARGET_ID= ; ",
                entity_id[i], seq_type[i], seq[i], chid[i]);
    }
    fprintf(fcgi,"\n");

    fclose(fcgi);

}


int seqres(char *inpfile)
{
    char ** line;
    int nline, i, key=0;

    line=get_lines_from_file(inpfile, &nline);
    for(i=0; i<nline; i++){
        if(!strncmp(line[i],"SEQRES ", 7) && strlen(line[i])>18){
            key=1;
            break;
        }else if (!strncmp(line[i], "ATOM", 4)|| !strncmp(line[i], "HETATM", 6)){
            break;
        }
        
    }
    if(nline>0) free_memory_2d(line, nline);
    return key;
}

void add_chain_to_pdb(char *inpfile)
{

    char  outfile[512], **line;
    int i, nch=0, nline=0, nter=0, nend=0, nmod=0, mod=0, natm=0;
    const char *chainid="ABCDEFGHIJKLMNOPQRSTUVWXYZ012345689abcdefghijklmnopqrstuvwxyz";
    FILE *fp=NULL;

    sprintf(outfile,"%s_new", inpfile);
    if((fp = fopen(outfile, "w"))==NULL) open_file_error(outfile, "add_chain_to_pdb");
    
    line=get_lines_from_file(inpfile, &nline);
    for(i=0; i<nline; i++){
        if (!strncmp(line[i], "TER", 3)){
            nter++;
        }else if (!strncmp(line[i], "ENDMDL", 6)){
            nmod++;
        }else if (!strncmp(line[i], "END", 3) && !strstr(line[i],"ENDMDL")){
            nend++;
        }
    }

    if(nter>0 && nend>0 && nmod==0 && !strcmp_case(METHOD, "NMR")){
        sprintf(INFO, "Warning: this is complicated case, please double check.\n");
        pinfo(INFO,0);
    }
    
    natm=0;
    mod=0;   
    for(i=0; i<nline; i++){
        if (!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ||
            !strncmp(line[i], "ANISOU", 6)){
            natm++;
            if (!strcmp_case(METHOD, "NMR") && nmod==0){ //add model/endmdl
                if (natm==1) {
                    fprintf(fp, "MODEL   1\n");
                    mod=1;
                }
            }
            
            line[i][21] = chainid[nch];
            
        }else if(!strncmp(line[i], "TER", 3) ||
                 (!strncmp(line[i], "END", 3) && !strstr(line[i],"ENDMDL")) ){
            nch++;
            if(nch>61){
                sprintf(INFO, "Warning! too many chains, the rest (>62) are added!\n");
                pinfo(INFO,0);
                nch=61;
            }
            if (!strcmp_case(METHOD, "NMR") && nmod==0){ //add model/endmdl
                mod++;
                if(!strncmp(line[i], "END", 3)) {
                    fprintf(fp, "ENDMDL \nMODEL   %d\n",mod);
                    continue;
                }  
            }        
        }else if(!strncmp(line[i], "ENDMDL", 6)) {
            nch=0;
        }
        fprintf(fp,"%s", line[i]);  
    }
    
    fclose(fp);    
    if(nline>0) free_memory_2d(line, nline);
    printf("\nChain ID has been assigned to the file %s\n\n", outfile);
    exit(0);
        
}

void output_info()
{
    printf("\nOutput file names:  \n");
    printf("    data_template.cif   (Author template file in mmCIF format).\n");
    printf("    data_template.text  (Author template file in plain text).\n");
    printf("    log_script.inp      (Command-like script file in plain text).\n");
            
    printf("\n------------------------------------------------------------------------\n");
    printf("Steps to assemble complete structure information for deposition.\n \
(1). Complete the template file data_template.text (or data_template.cif).\n \
(2). Either\n \
     use the pdb_extract command line (type 'pdb_extract -h' for help).\n \
     Or\n \
     Edit log_script.inp and run the command 'extract -ext log_script.inp'\n");
    printf("------------------------------------------------------------------------\n");

}

int  get_pdb_one_model(char *inpfile,  char *outfile)
/* If multi-model exist, only get the first one! 
*/
{
    char str[200], str_new[200];
    int key=0, n=0;
    FILE *fp, *fo;
    
    
    if((fp = fopen(inpfile, "r"))==NULL){
        open_file_error(inpfile, "get_pdb_one_model");
    }
    
    fo = fopen(outfile, "w");

    while (fgets(str, sizeof str, fp) != NULL) {
            //NMR
        if(!strncmp(str,"SEQRES", 6) && strlen(str)>24){
            key=1;
        }

        if (!strcmp(METHOD, "NMR") && (!strncmp(str, "ATOM", 4) ||
                                       !strncmp(str, "HETATM", 6))){
            str[20]=' '; 
        }
        
        if(!strncmp(str, "ENDMDL", 6)) break;
        
        n=replace_na_name(str, str_new);
        if (n==1) strcpy(str, str_new); 
            
        fprintf(fo,"%s", str);
    }
    fclose(fo);
    fclose(fp);

    return key;
}

void usage(void)
{
    printf("\nUsage: extract [option] input_file\n");
    printf("-PDB  followed by a coordinate file in PDB format.\n");
    printf("-CIF  followed by a coordinate file in mmCIF format.\n");
    printf("-EXT  followed by the file log_script.inp (generated).\n");
    printf("-CHAIN  followed by a pdb file (add chain ID for PDB file).\n");
    printf("-SOL  followed by data_template.text to update matth_coeff.\n\n");
    printf("-NMR  If added, it is for NMR experiment.\n");
    printf("-EM   If added, it is for EM experiment. \n");
        
    printf("EXAMPLE for X-RAY structures:\n");
    printf("  Type 'extract  -PDB  pdb_file_name' \n");
    printf("EXAMPLE for NMR structures:\n");
    printf("  Type 'extract  -PDB  pdb_file_name -NMR' \n\n");
    
    printf("Enter necessary information to data_template.text and log_script.inp\n");
    printf("\nEITHER\n");
    printf("use 'extract  -EXT log_script.inp ' to get a complete mmCIF output.\n");
    printf("OR\n");
    printf("use pdb_extract (type pdb_extract -h for help) to get a complete mmCIF output.\n\n");
    
    exit(0);
}


void remove_ligand_in_seq(int i, int n, int**ch_idx,char**resname,char **chain,
                          int **res_idx, char **atomname, float **xyz)
//remove 'ligand' in the end of the sequence.
//i, chain number; n, polymer type
{
    int m=0, c3p, c1p,  n1, n2=-1, j;
    
    for (j=ch_idx[i][1]-1; j>=ch_idx[i][0]; j--){ 
        
        if(!protein_na_residue(resname[j]) ){
             
            if (n==2 || n==3){ //protein
                if ((!strcmp(resname[j], "NH2")||!strcmp(resname[j], "ACE"))
                    && j>0 && is_protein(resname[j-1])) continue;
            
                n1=find_atom(" CA ", atomname, res_idx[j][0],  res_idx[j][1]);
                if (j>0) n2=find_atom(" CA ", atomname, res_idx[j-1][0],  res_idx[j-1][1]);
                if ((n1>0 && !strcmp(resname[j], "UNK")) ||
                    (j>0 && n1>=0 && n2>=0 && dist(xyz[n1],xyz[n2])<4.5) ||
                    (j==0 && n1>0)) continue;
                
            }else{ //na
//                printf("nch== %d %d %d  %d %s  %d %d %d %s \n", i, n, m, j, resname[j],ch_idx[i][1], ch_idx[i][0], res_idx[j][0],  chain[j]);
                c3p = find_atom(" C3'", atomname, res_idx[j][0],  res_idx[j][1]);
                if (c3p<0) c3p = find_atom(" C3*", atomname, res_idx[j][0],  res_idx[j][1]);
                
                c1p = find_atom(" C1'", atomname, res_idx[j][0],  res_idx[j][1]);
                if (c1p<0) c1p = find_atom(" C1*", atomname, res_idx[j][0],  res_idx[j][1]);
                
                n1=find_atom(" P  ", atomname, res_idx[j][0],  res_idx[j][1]);
                if (j>0) n2=find_atom(" P  ", atomname, res_idx[j-1][0],  res_idx[j-1][1]);
                
                if (c3p>=0 && c1p>=0 &&
                    (j==0 || (j>0 && n1>=0 && n2>=0 && dist(xyz[n1],xyz[n2])<7.5))) continue;
            }
            m++;
        }else {
            break;
        } 
    }
    if (m>0 && m==ch_idx[i][1]-ch_idx[i][0]){
        printf("Note: chain (%s) is non-polymer (nres=%d)! \n", chain[ch_idx[i][0]], m);
        return;
    }
    
    ch_idx[i][1]=ch_idx[i][1]-m;
}


void get_bseq(int n, int j, int **res_idx, char **resname, char *bseq)
// assign 3 letter residue to 1 letter residue!
{
    int i;

    for (i =res_idx[j][0] ; i<res_idx[j][1]; i++) {
   
        if(n==0){//rna/dna
            if      (!strcmp_case(resname[i],"A") || !strcmp_case(resname[i],"ADE") ||
                     !strcmp_case(resname[i],"DA"))
                bseq[i] = 'A';
            else if (!strcmp_case(resname[i],"G") || !strcmp_case(resname[i],"GUA") ||
                     !strcmp_case(resname[i],"DG"))
                bseq[i] = 'G';
            else if (!strcmp_case(resname[i],"U") || !strcmp_case(resname[i],"URA") ||
                     !strcmp_case(resname[i],"URI"))
                bseq[i] = 'U';
            else if (!strcmp_case(resname[i],"C") || !strcmp_case(resname[i],"CYT") ||
                     !strcmp_case(resname[i],"DC"))
                bseq[i] = 'C';
            else if (!strcmp_case(resname[i],"T") || !strcmp_case(resname[i],"THY") ||
                     !strcmp_case(resname[i],"DT"))
                bseq[i] = 'T';
            else 
                bseq[i] = 'X';

            
        }else if(n==1){ //Protein
            
/* Hydrophobic  amino acids */
            if      (!strcmp_case(resname[i],"LEU"))
                bseq[i] = 'L';
            else if (!strcmp_case(resname[i],"ILE"))
                bseq[i] = 'I';
            else if (!strcmp_case(resname[i],"ALA"))
                bseq[i] = 'A';
            else if (!strcmp_case(resname[i],"VAL"))
                bseq[i] = 'V';
            else if (!strcmp_case(resname[i],"PHE"))
               bseq[i] = 'F';
            else if (!strcmp_case(resname[i],"PRO"))
               bseq[i] = 'P';
            else if (!strcmp_case(resname[i],"MET"))
               bseq[i] = 'M';

/* chared amino acids */
            else if (!strcmp_case(resname[i],"ASP"))
               bseq[i] = 'D';
            else if (!strcmp_case(resname[i],"GLU"))
               bseq[i] = 'E';
            else if (!strcmp_case(resname[i],"LYS"))
               bseq[i] = 'K';
            else if (!strcmp_case(resname[i],"ARG"))
               bseq[i] = 'R';

/* Polar amino acids */
            else if (!strcmp_case(resname[i],"SER"))
               bseq[i] = 'S';
            else if (!strcmp_case(resname[i],"THR"))
               bseq[i] = 'T';
            else if (!strcmp_case(resname[i],"TYR"))
               bseq[i] = 'Y';
            else if (!strcmp_case(resname[i],"HIS"))
               bseq[i] = 'H';

            else if (!strcmp_case(resname[i],"CYS"))
               bseq[i] = 'C';
            else if (!strcmp_case(resname[i],"ASN"))
               bseq[i] = 'N';
            else if (!strcmp_case(resname[i],"GLN"))
               bseq[i] = 'Q';
            else if (!strcmp_case(resname[i],"TRP"))
               bseq[i] = 'W';

/* Glycine */
            else if (!strcmp_case(resname[i],"GLY"))
                bseq[i] = 'G';
            else 
                bseq[i] = 'X';
        }else { //ligand
            bseq[i] = 'X';
        }
        
            
    }
    
}


void  three2one(char *resname, char *one)
// assign 3 letter residue to 1 letter residue!
{
     char bseq;
   
            if      (!strcmp_case(resname,"A") || !strcmp_case(resname,"ADE") ||
                     !strcmp_case(resname,"DA"))
                bseq = 'A';
            else if (!strcmp_case(resname,"G") || !strcmp_case(resname,"GUA") ||
                     !strcmp_case(resname,"DG"))
                bseq = 'G';
            else if (!strcmp_case(resname,"U") || !strcmp_case(resname,"URA") ||
                     !strcmp_case(resname,"URI"))
                bseq = 'U';
            else if (!strcmp_case(resname,"C") || !strcmp_case(resname,"CYT") ||
                     !strcmp_case(resname,"DC"))
                bseq = 'C';
            else if (!strcmp_case(resname,"T") || !strcmp_case(resname,"THY") ||
                     !strcmp_case(resname,"DT"))
                bseq = 'T';

            
            
/* Hydrophobic  amino acids */
            else if (!strcmp_case(resname,"L") || !strcmp_case(resname,"LEU"))
                bseq = 'L';
            else if (!strcmp_case(resname,"I") || !strcmp_case(resname,"ILE"))
                bseq = 'I';
            else if (!strcmp_case(resname,"A") || !strcmp_case(resname,"ALA"))
                bseq = 'A';
            else if (!strcmp_case(resname,"V") || !strcmp_case(resname,"VAL"))
                bseq = 'V';
            else if (!strcmp_case(resname,"F") || !strcmp_case(resname,"PHE"))
               bseq = 'F';
            else if (!strcmp_case(resname,"P") || !strcmp_case(resname,"PRO"))
               bseq = 'P';
            else if (!strcmp_case(resname,"M") || !strcmp_case(resname,"MET"))
               bseq = 'M';

/* chared amino acids */
            else if (!strcmp_case(resname,"D") || !strcmp_case(resname,"ASP"))
               bseq = 'D';
            else if (!strcmp_case(resname,"E") || !strcmp_case(resname,"GLU"))
               bseq = 'E';
            else if (!strcmp_case(resname,"K") || !strcmp_case(resname,"LYS"))
               bseq = 'K';
            else if (!strcmp_case(resname,"R") || !strcmp_case(resname,"ARG"))
               bseq = 'R';

/* Polar amino acids */
            else if (!strcmp_case(resname,"S") || !strcmp_case(resname,"SER"))
               bseq = 'S';
            else if (!strcmp_case(resname,"T") || !strcmp_case(resname,"THR"))
               bseq= 'T';
            else if (!strcmp_case(resname,"Y") || !strcmp_case(resname,"TYR"))
               bseq = 'Y';
            else if (!strcmp_case(resname,"H") || !strcmp_case(resname,"HIS"))
               bseq = 'H';

            else if (!strcmp_case(resname,"C") || !strcmp_case(resname,"CYS"))
               bseq = 'C';
            else if (!strcmp_case(resname,"N") || !strcmp_case(resname,"ASN"))
               bseq = 'N';
            else if (!strcmp_case(resname,"Q") || !strcmp_case(resname,"GLN"))
               bseq= 'Q';
            else if (!strcmp_case(resname,"W") || !strcmp_case(resname,"TRP"))
               bseq = 'W';

/* Glycine */
            else if (!strcmp_case(resname,"G") || !strcmp_case(resname,"GLY"))
                bseq = 'G';
            else if (!strcmp_case(resname,"SEC")) //Selenocysteine
                bseq = 'U';
            else if (!strcmp_case(resname,"PYL")) //pyrrolysine
                bseq = 'O';
            else 
                bseq = 'X';
      one[0]= bseq;
      one[1]='\0';
}



void get_template_files(char *adit_text, char *adit_inp)
{
    char *temp=NULL, text_path[512];
    
    strcpy(adit_text, "");
    strcpy(adit_inp, "");
    (!strcmp_case(CCP4,"yes")) ? temp = getenv("CCP4"):temp = getenv("PDB_EXTRACT");
        
    if(temp == NULL) {
        if(!strcmp_case(CCP4,"yes"))
            printf("Wrong! You did not set CCP4 environment.\n");
        else
            printf("Wrong! You did not set PDB_EXTRACT environment.\n");
        printf("Please set PDB_EXTRACT environment according to the README instruction in the package.\n");
        return;
    }else{  
        strcpy(text_path, temp);
        
        if(!strcmp_case(CCP4,"yes")){
            strcat(text_path,CCP4_2_DATA);
        }else{
            strcat(text_path,EXTRACT_2_DATA);
        }
        strcat(adit_text,text_path);
        strcat(adit_inp, text_path);

        
        if(!strcmp_case(METHOD, "NMR")){
            strcat(adit_text,"nmr_data_template.txt");
            strcat(adit_inp, "nmr_log_script.inp");
        }else if (!strcmp_case(METHOD, "EM")){
            strcat(adit_text,"em_data_template.txt");
            strcat(adit_inp, "em_log_script.inp");
        }else{
            strcat(adit_text,"data_template.text");
            strcat(adit_inp, "log_script.inp");
        }
    }
}

void type_of_polymer(int nres, char **resname, char * str)
//protein 1, rna/dna=2, rna=3, dna=4
{
    int j, key=0, dna=0, rna=0;
     
    static const char *protein20[]={
        "LEU","PRO","ALA","VAL","GLY","SER","ASN","GLN","ASP","GLU","MET","TRP","PHE",
        "ILE","THR","CYS","TYR","HIS","LYS","ARG"
    };
    
    static const char *rna3[]={
        "U", "URA", "URI"
    };

    static const char *dna14[]={
        "ADE","GUA","CYT","THY","A","G","T","C","I", "DA","DG","DT","DC","DI"
    };

    for (j=0; j<nres; j++){
        key=is_protein_na_residue(20, resname[j], protein20);
        if(key>0){
            strcpy(str, " polypeptide(L) ");
            key=1;
            return ; //protein
        }
    }
    
    for (j=0; j<nres; j++){
        key=is_protein_na_residue(3, resname[j], rna3);
        if(key>0){
            rna=1; //rna
//            printf("key=%d %d %s \n", key, j, resname[j]);
            
            break;
        }
    }
    
    for (j=0; j<nres; j++){
        if (!strcmp(resname[j],"DA") || !strcmp(resname[j],"DG") ||
            !strcmp(resname[j],"DT") || !strcmp(resname[j],"DC")){
            dna=1;
            break;
        }
    }
    if (rna && dna){
        strcpy(str, " polydeoxyribonucleotide/polyribonucleotide hybrid ") ;
        key=2;
        return ; //dna/rna hybrid
    }else if (rna){
        strcpy(str, " polyribonucleotide ");
        key=3;
        return ; //rna only
    }
    

    for (j=0; j<nres; j++){
        key=is_protein_na_residue(14, resname[j], dna14);
        if(key>0){
            strcpy(str, " polydeoxyribonucleotide ") ;
            key=4;
            return ; //dna
        }
    }
    strcpy(str, " other ");
    
}


int **chain_break(int nch, int **ch_idx, int **res_idx, char **atomname, float**xyz)
//ch_idx :  begin/end residue number for the chain.
//res_idx:  begin/end atom number for the residue.  
{
    int p1=-1,p2=-1, c3p=-1, o3p=-1, o5p=-1, c=-1, ca1=-1, ca2=-1;
    int i=0, j=0, n1, n2, m1, m2, k1, k2, **ch_break;
    float dis;
    

    ch_break=new int *[nch+1];
    for (i=0; i<nch; i++){ 
        k1=ch_idx[i][0];
        k2=ch_idx[i][1];
        ch_break[i]=new int[k2+3];
    }
    
    for (i=0; i<nch; i++){
         for (j=ch_idx[i][0]; j<ch_idx[i][1]; j++) ch_break[i][j]=1;
    }

    for (i=0; i<nch; i++){ //through chainID
        k1=ch_idx[i][0];
        k2=ch_idx[i][1];
        
        for (j=k1; j<k2-1; j++){  //through residue
            n1=res_idx[j][0];
            n2=res_idx[j][1];
            m1=res_idx[j+1][0];
            m2=res_idx[j+1][1];
            
            c=find_atom(" C  ", atomname, n1,n2);
            ca1=find_atom(" CA ", atomname, n1,n2);
            ca2=find_atom(" CA ", atomname, m1,m2);
         
            if (ca1>=0 && ca2>=0 ){ //protein
                dis=dist(xyz[ca1],xyz[ca2]);
                if (dis > 4.5) {
                    ch_break[i][j]=0;
//                    printf("break= %3d  %3d  %3d  %3d  : %3d  %3d  %3d  %3d  : %3d  %3d %.2f \n", k1,k2,i, j, n1,n2,m1,m2, ca1,ca2, dis);
                }
                
                continue;
            }else if (c>=0 && ca2>=0 ){
                if (dist(xyz[c],xyz[ca2])> 4.5) ch_break[i][j]=0;
                continue;
            }else{ //na
                p1  = find_atom(" P  ", atomname, n1, n2);
                p2  = find_atom(" P  ", atomname, m1, m2);
                
                if (is_prime(atomname, n1, n2)){
                    c3p = find_atom(" C3'", atomname, n1, n2);
                    o3p = find_atom(" O3'", atomname, n1, n2);
                    o5p = find_atom(" O5'", atomname, m1, m2);
                }else{
                    c3p = find_atom(" C3*", atomname, n1, n2);
                    o3p = find_atom(" O3*", atomname, n1, n2);
                    o5p = find_atom(" O5*", atomname, m1, m2);
                }
                
                    
                
                if (c3p>=0 && p2>=0){
                    if (dist(xyz[c3p],xyz[p2])> 4.5) { 
                    ch_break[i][j]=0;
                    //printf("break(c3p-p2)= %3d  %3d  %3d  %3d  : %3d  %3d  %3d  %3d  : %3d  %3d %.2f \n", k1,k2,i, j, n1,n2,m1,m2, c3p,p2, dist(xyz[c3p],xyz[p2]));
                    }                    
                    continue;
                }else if (o3p>=0 && o5p>=0){
                    if (dist(xyz[o3p],xyz[o5p])> 4.8){ ch_break[i][j]=0;
                   // printf("break(o3p-o5p)= %3d  %3d  %3d  %3d  : %3d  %3d  %3d  %3d  : %3d  %3d %.2f \n", k1,k2,i, j, n1,n2,m1,m2, o3p,o5p, dist(xyz[o3p],xyz[o5p]));
                    }
                    continue;
                }else if (p1>=0 && p2>=0){
                    if (dist(xyz[p1],xyz[p2])> 8.5){ ch_break[i][j]=0;
                   // printf("break(p1-p2)= %3d  %3d  %3d  %3d  : %3d  %3d  %3d  %3d  : %3d  %3d %.2f \n", k1,k2,i, j, n1,n2,m1,m2, o3p,o5p, dist(xyz[p1],xyz[p2]));
                    }
                    continue;
                }
                
            }
        }
    }
    return ch_break;
}

char **get_all_atoms(char *inpfile, int *nline)
//Only for all the atoms .
{
    char str[5000], **all_line=NULL, info[512];
    int  m=0;
    FILE *fp=NULL;

    *nline=0;
    if((fp=fopen(inpfile, "r"))==NULL){
        sprintf (info, "Can not open file (%s) in get_all_atoms.\n", inpfile);
        pinfo(info,0);
        return NULL;
    }
    
    m=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if (!strncmp(str, "ATOM  ", 6) || !strncmp(str, "HETATM", 6)) m++;
    }
    
    rewind(fp);
    all_line=new char *[m+1];

    m=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if (!strncmp(str, "ATOM  ", 6) || !strncmp(str, "HETATM", 6)){
            if (!strncmp(str+17,"HOH",3)) continue;

            all_line[m] = new char [strlen(str) +1];
            strcpy(all_line[m],str);
            m++;
        }
    }
    *nline = m;
    fclose(fp);
    return (all_line);
}


void get_matched_poly(int nch, int *ptype, int **ch_idx, char *bseq, char **chain,
                     int *best_nseq, char **matched_ch, int *poly)
{
   // char chain_nam1[4], chain_nam2[4], entity_name[80];
    char **chain_res;
    int i,j, k, n,m, *matched_comp, *matched_num, *chlen, npoly=0;
    long nmatch=0;
    double percent=0;


    printf("Match criteria: similarity=100%% for protein; =100%% for NA.\n"); //chenghua-20190322

    
    strcpy(matched_ch[0], "");
    *poly=0;
    chain_res=new char *[nch+10]; //residue held in the chain.
    for (i=0; i<nch; i++){
        n=ch_idx[i][1]-ch_idx[i][0];
        chain_res[i]=new char [n+3]; 
        j=0;
        for (k=ch_idx[i][0]; k<ch_idx[i][1]; k++){
            chain_res[i][j]=bseq[k];
            j++;
        }
        chain_res[i][j] = '\0';
        
     //   printf("i,ch, seq= %2d %2s %4d %4d %s\n", i, chain[ch_idx[i][0]],ch_idx[i][0], ch_idx[i][1], chain_res[i]);
    }
    
    npoly=0;
    matched_comp = new int [nch];
    matched_num=new int [nch];
    chlen=new int[nch+2];
    for (k=0; k<nch; k++) {
        matched_num[k]=-1;
        matched_comp[k]=-1;
        chlen[k]=ch_idx[k][1] - ch_idx[k][0];
    }
    
    for (i=0; i<nch; i++){
        if(ptype[i]== -1)continue; /*rid of ligand*/
        for (k=0; k<nch; k++){ matched_num[k]=-1;} //initialize again
        matched_num[i]=i;
        n=0;
        for (k=0; k<nch; k++) {
            if (i==matched_comp[k]){
                n=1;
                break;
            }
        }
        if (n==1) continue; /* matched with this chain*/
       // printf("chaa1=%3d %3d %3d  %3d\n", i,ch_idx[i][0], ch_idx[i][1],chlen[i]);

        k=0;
        for (j=i+1; j<nch; j++){
            if(ptype[i] != ptype[j]) continue;
            if (matched_comp[j]>=0)  continue; //already matched!
            
            seq_align(i, j, chain_res, &percent, &nmatch);
            
//      fasta_seq_align(i, j, chain_res, &score);
            
	    //chenghua-20190322-start
	    //change matching creterion from 80%(protein)/95%(NA) to 100% for all
            if((((ptype[i]==0 && ptype[j] ==0)|| (ptype[i]==1 && ptype[j] ==1) ||
                 (ptype[i]==4 && ptype[j] ==4)) && percent>99.99 )|| //Nucleic acids 
	       ((ptype[i]==2 || ptype[i]==3) && (ptype[j]==2 || ptype[j]==3) && percent>99.99 )) { //protein
	      //chenghua-20190322-end
	      matched_comp[j]=j;
	      matched_num[j]=j;
	      printf("Sequence similarity(%s:%s)=%.1f%%; nmatch=%2ld\n", chain[ch_idx[i][0]],chain[ch_idx[j][0]], percent, nmatch);                    
            }
        }

        n=0;
        m=i;
        
        for (k=0; k<nch; k++){ //get the longest length of the chain.
            if (matched_num[k]>=0) {
                if (chlen[k]>n){
                    n =  chlen[k];
                    m = matched_num[k];
                }
                strcat(matched_ch[npoly], chain[ch_idx[k][0]]);
                strcat(matched_ch[npoly], ",");
            }
        }
        
        
        rid_of_space(matched_ch[npoly]);
        n=strlen(matched_ch[npoly]); 
        if (matched_ch[npoly][n-1]==',') matched_ch[npoly][n-1]='\0';
        
        
        best_nseq[npoly]=m;
//        printf("Number of best matched chains = %d (%s), len=%d (%d - %d)\n\n",m, matched_ch[npoly], chlen[m], ch_idx[m][0], ch_idx[m][1]);
        npoly++;
    }
    *poly=npoly;

    
}

void align_use_coord(char *pdbfile)
//sequences alignment from the coordinate
{
    char **line, **chain, **resname, **atomname, *bseq;
    char **poly_type, **matched_ch ;
    int i, n=0, nline, npoly,  nch_tmp=0, nres_tmp=0, nch, nres;
    int **res_idx, **ch_idx, *ptype, **pptype,*best_nseq, **ch_break;
    float **xyz;


    printf("\nDo alignment using the sequence from coordinate.\n");
    
    line=get_lines_from_file(pdbfile, &nline);
    get_nchain_nres(nline, line, &nres_tmp, &nch_tmp);
    xyz=set_fmemory_2d(nline, 4);
    atomname=set_memory_2d(nline, 4);
    ch_idx=set_imemory_2d(nch_tmp +40, 3);
    res_idx=set_imemory_2d(nres_tmp+4, 2);
    resname=set_memory_2d(nres_tmp+2,3);
    chain=set_memory_2d(nres_tmp+2,4); // chain and 2 charactor chainID

    atominfo(nline,line,atomname,resname,chain, xyz, res_idx, &nres, ch_idx, &nch);
    if (nch<=0){
        sprintf(INFO, "Error: Chain ID is not identified (check pdb format).\n" );
        pinfo(INFO,0);
        return;
    }
        
    bseq=new char [nres+2];
    ptype=new int [nch+2];//number of entity
    poly_type=set_memory_2d(nch, 80);
    pptype=set_imemory_2d(nch, nres);
    for (i=0; i<nch; i++){ //n=0 rna; 1 dna, n=2,3 prot, n=4 rna/dna; n=-1 other
        n=protein_or_na(i, ch_idx, resname, res_idx, atomname); 
        ptype[i]=n;
        if (n>=0) {//modified ch_idx (last residues), remove ligand
            remove_ligand_in_seq(i,n, ch_idx, resname, chain, res_idx, atomname,xyz);
        }
        get_poly_type(n,i, ch_idx, res_idx, atomname, pptype, poly_type);

        if(n==0 || n==1 || n==4){ //rna/dna
            get_bseq(0, i, ch_idx, resname, bseq);
        }else if(n==2){ //protein
            get_bseq(1,i, ch_idx, resname, bseq);
        }else{
            get_bseq(-1,i, ch_idx, resname, bseq);
            sprintf(INFO, "Note: all non-polymer for chain (%s : %d->%d).\n",
                    chain[ch_idx[i][0]],ch_idx[i][0],ch_idx[i][1] );
            pinfo(INFO,1);
        }
//        printf("chain, ptype, %s %3d %3d %s\n", chain[ch_idx[i][0]], i, n, poly_type[i]);
    }
    
    int j, n1,n2, dn1,dn2;
    for (i=0; i<nch-1; i++){
        n1=ch_idx[i][0];
        if (ptype[i]<0) continue;
        
        for (j=i+1; j<nch; j++){
            if (ptype[j]<0) continue;
            n2=ch_idx[j][0];
            if (!strcmp(chain[n1], chain[n2])){
                dn1=ch_idx[i][1] - ch_idx[i][0];
                dn2=ch_idx[j][1] - ch_idx[j][0];
                if (dn1>dn2){
                    ch_idx[j][1]=ch_idx[j][0];
                }else{
                    ch_idx[i][1]=ch_idx[i][0];
                    
                }
                
                sprintf(INFO,"Warning: duplicate chains removed.(nch=%d vs nch=%d) (%d %d) %s\n", i,j,dn1,dn2, chain[n1]);
                pinfo(INFO,0);
                
            }
        }
    }
    

    ch_break=chain_break(nch, ch_idx, res_idx, atomname, xyz);//ch_break[nch][nres]
    

/*    
    for (i=0; i<nch; i++){ //through chainID
        for (j=ch_idx[i][0]; j<ch_idx[i][1]; j++){  //through residue
            printf("out= %3d %4d %4d   %4d %4d %4d   %d %2s %4s %c\n",
                   i, ch_idx[i][0], ch_idx[i][1], j, res_idx[j][0], res_idx[j][1],
                   ch_break[i][j], chain[j], resname[j], bseq[j]);
        }
    }
*/
    best_nseq=new int[nch+1];
    matched_ch=set_memory_2d(nch+1, 2*nch);

    for (i=0; i<nch+1; i++) strcpy(matched_ch[i], "");
    
    get_matched_poly(nch, ptype, ch_idx, bseq, chain,best_nseq, matched_ch, &npoly);

    write_poly(npoly, poly_type, best_nseq, matched_ch,atomname, resname,bseq,
               pptype, ch_idx, res_idx, ch_break);
    printf("Number_total_residues=%d; total segment of chains=%d. \n", nres, nch);

     
}


void get_seqres(int start, int nline, char **line, char *ch_id,  int *res, char **resname)
//get residue from the seqres for the ch_id;
{
    char **tokens, chain[4], str[200];
    int i,j, n=0,nres=0,ntok;
    
    for (i=start; i<nline; i++){
        if (!strncmp(line[i], "SEQRES", 6)) {
            strncpy(chain, line[i]+10, 2);
            chain[2]='\0';
            rid_of_space(chain);
            if (!strcmp(chain, ch_id)){
                strcpy(str,line[i]+18);
                tokens=string_token(str, " ", &ntok);
                for (j=0; j<ntok; j++){
                    if(j>12 || strlen(tokens[j])>3) continue;//fixed 13 residues 
                    strcpy(resname[nres], tokens[j]);
                    nres++;
                }
                if(ntok>0) free_memory_2d(tokens,ntok);
            }else if( nres && strcmp(chain, ch_id)) {
                break;
            }
            
            n++;
        }
    }
    *res=nres;
    
}

void get_ch_match(int n, int nline, char **lines,  char *str)
{
    char ss[200];
    int i;
    
    
    strcpy(str,"");
    for(i=n; i<nline; i++){
        strcpy(ss,lines[i]);
        replace_char(ss, '\n', ' ');
        if(i>n && (strstr(ss, ": ") || i-n>4)) break;
        if(strstr(ss, ": ")) {
            strcat(str, strstr(ss, ": ")+1);
        }else{
            strcat(str,ss + 10);
        }
    }
//        printf("hello=%d %d  (%s)\n", n, nline, str);
    
    if (strlen(str)>0){
        clean_string(str,str);
        replace_char(str, ';', ' ');
    }
    
}
    
int align_use_seqres(char *pdbfile)
//No alignment required, use MOL_ID, CHAIN, and SEQRES to get the entity poly
{
    char **resname, **tokens, **line, *str_seq,one[3], ch_id[80], mod_id[80], tmp[300];
    char **seq_match, **poly_match, **ch_match;
    int  nline, i, j,k,id=0, succ=1,  nres=0,  ntok, npoly=0;

    static const char *residues26[]={
        "LEU","PRO","ALA","VAL","GLY","SER","ASN","GLN","ASP","GLU",
        "MET","TRP","PHE","ILE","THR","CYS","TYR","HIS","LYS","ARG",
        "ADE","GUA","CYT","THY","URI","URA"
    };
    printf("\nDo alignment using the sequence from SEQRES.\n");
    
    line=get_lines_from_file(pdbfile, &nline);
    resname= set_memory_2d(90000, 5);

    ch_match=new char *[1000];
    seq_match=new char *[1000];
    poly_match=new char *[1000];
    
    for (i=0; i<nline-1; i++){ //estimate number of residues
        if(!strncmp(line[i], "COMPND", 6) && strstr(line[i], " MOL_ID: ")
           && strchr(line[i],';')){
            
            get_data_set_id(line[i], ":", ";", mod_id);
            rid_of_space(mod_id);
            
            strcpy(tmp,"");
            for (j=i+1; j<nline; j++){
                if (!strncmp(line[j], "COMPND", 6) && strstr(line[j], " CHAIN: ")){
                    get_ch_match(j, nline, line, tmp);

                    i=j-1;
                    break;
                }
            }
            rid_of_space(tmp);
            
            if (!strlen(tmp) || !(isdigit(mod_id[0]))) {
                succ=0; //not successful                
                break;
            }
            
            ch_match[npoly]=new char[strlen(tmp)+1];
            strcpy(ch_match[npoly], tmp);

            tokens=string_token(tmp, ",", &ntok);
            strcpy(ch_id, tokens[0]);
            rid_of_space(ch_id);
            get_seqres(j, nline, line, ch_id, &nres, resname);
            if (!nres) succ=0;
            
            type_of_polymer(nres, resname, tmp);
            poly_match[npoly]=new char[strlen(tmp) +1];
            strcpy(poly_match[npoly], tmp);
            
            if(ntok>0) free_memory_2d(tokens,ntok);
            str_seq =new char [nres*4+1];
            strcpy(str_seq, "");
            for (j=0; j<nres; j++){
                if(strlen(resname[j])==1){
                    strcat(str_seq, resname[j]);
                }else{

                    id=0;
                    for(k=0;k<26;k++){
                       if(!strcmp(residues26[k], resname[j])){
                           id=1;
                           break;
                        }
                    }
                    if (id){
                        three2one(resname[j], one);
                        strcat(str_seq, one);
                    }else{
                        strcat(str_seq,"(");
                        strcat(str_seq, resname[j]);
                        strcat(str_seq, ")");
                    }
                }
//                printf("result %2d %2d %s %s\n",j, nres,ch_id, resname[j]);
            }
            seq_match[npoly]=new char[strlen(str_seq)+1];
            strcpy(seq_match[npoly], str_seq);
            if(strlen(str_seq)>0) delete []str_seq;

            npoly++;
            
        }
    }
/*    
    for (i=0; i<npoly; i++){
        printf("final %2d %s %s %s\n",i, poly_match[i],ch_match[i],seq_match[i]);
    }
*/
    if (succ && npoly) {
        write_poly1(npoly, poly_match,ch_match,seq_match);
        return 1;
    }else{
        return 0;
    }
    
}
void write_poly_from_cif(char *iFile)
{
    char **etype, **estr, **eseq;
    string diags, blockId;
    long n1=0, i;

    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;

    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(iFile, diags);
    delete(cifParserR);

    if (!diags.empty()) {
        fprintf(stderr, "Diags for file %s: %s\n",iFile,diags.c_str());
    }

    blockId = fobjR->GetFirstBlockName();

    etype = get_attribute_values(fobjR, blockId.c_str(), "entity_poly", "type", &n1);
    eseq = get_attribute_values(fobjR, blockId.c_str(), "entity_poly", "pdbx_seq_one_letter_code", &n1);
    estr = get_attribute_values(fobjR, blockId.c_str(), "entity_poly", "pdbx_strand_id", &n1);
    
    if (get_attribute_value(fobjR, blockId.c_str(), "cell", "length_a")){
        strcpy(cell.a, get_attribute_value(fobjR, blockId.c_str(), "cell", "length_a"));
        strcpy(cell.b, get_attribute_value(fobjR, blockId.c_str(), "cell", "length_b"));
        strcpy(cell.c, get_attribute_value(fobjR, blockId.c_str(), "cell", "length_c"));
        strcpy(cell.alpha, get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_alpha"));
        strcpy(cell.beta, get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_beta"));
        strcpy(cell.gamma, get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_gamma"));
        strcpy(cell.space_group, get_attribute_value(fobjR, blockId.c_str(), "symmetry", "space_group_name_H-M"));
        
        fprintf(fcgi, "a=%s:b=%s:c=%s:alpha=%s:beta=%s:gamma=%s:space_group=\"%s\"\n",
                cell.a, cell.b, cell.c, cell.alpha, cell.beta, cell.gamma, cell.space_group); 
    }
    if (n1) {
        for(i=0; i<n1; i++){
            replace_char(eseq[i], '\n', ' ');
            replace_char(estr[i], '\n', ' ');
        }
        write_poly1(n1, etype, estr, eseq);
    }else{
        sprintf(INFO, "Warning: The input cif file (%s) has no entity_poly category.\n",iFile);
        pinfo(INFO, 0);
    }
}

void type_of_entity(int *ptype, int i, int **ch_idx, char *bseq, char *entity_name)
{
    char info[80];
    
    if(ptype[i] ==2 ){
        strcpy(entity_name," polypeptide(L) "); 
    }else if(ptype[i] ==0 ){
        strcpy(entity_name," polyribonucleotide "); 
    }else if (ptype[i] ==1 ){
        strcpy(entity_name," polydeoxyribonucleotide ") ;
    }else{
        sprintf(info, "Warning: polymer type (n=%d) can not be determined.\n", i);
        pinfo(info,0);
        strcpy(entity_name," ? ");
    }
}

void write_poly_infor(FILE *fout, int idd)
//idd==0: cif token only; ==1: 
{
    if (idd==0){
        fprintf(fout,"loop_\n");
        fprintf(fout,"_entity_poly.entity_id\n");
        fprintf(fout,"_entity_poly.type\n");
        fprintf(fout,"_entity_poly.pdbx_seq_one_letter_code\n");
        fprintf(fout,"_entity_poly.pdbx_strand_id\n");
        fprintf(fout,"_entity_poly.pdbx_seq_db_id\n");  //new
        fprintf(fout,"_entity_poly.pdbx_seq_db_name\n"); //new
        
    }else if (idd==1){
        fprintf(fout, "< target_DB_id=  > (if known) \n");
        fprintf(fout, "< sequence_database_id=  > (if known) \n"); //new
        fprintf(fout, "< sequence_database_name=  > (if known) \n\n");//new
        
    }else if (idd==2){
        fprintf(fout,"<molecule_entity_id=  >\n");
        fprintf(fout, "<molecule_entity_type=  >\n");            
        fprintf(fout, "<molecule_one_letter_sequence= >\n");
        fprintf(fout, "<molecule_chain_id=  >\n");
        fprintf(fout, "<target_DB_id=  >  (if known)\n");
        fprintf(fout, "<sequence_database_id=   > (if known) \n"); //new
        fprintf(fout, "<sequence_database_name=   > (if known) \n\n");//new
        
    }else if (idd==3){
        fprintf(fout, " TARGET_DB= :");
        fprintf(fout, " SEQUNCE_DB= :");
        fprintf(fout, " SEQUNCE_NAME= ;");
        
    }else if (idd==4){
        fprintf(fout, " ENTITY_ID = :");    
        fprintf(fout, " ENTITY_TYPE= :");
        fprintf(fout, " ONE_LETTER_SEQUNCE= :"); //also write to "cgi_value" file, add to end
        fprintf(fout, " CHAIN_ID= :");
        fprintf(fout, " TARGET_DB= :");
        fprintf(fout, " SEQUNCE_DB= :");
        fprintf(fout, " SEQUNCE_NAME= ;");
        
    }
}

void write_poly1(int npoly, char **poly_type, char **ch_match, char **seq_match) 
/*
  npoly : number of entity_poly;
  poly_type : type of polymers
  ch_match: combination of chains for the polymer
  seq_match: the representative sequence of the polymer
*/
{
    char adit_text[MAXL],adit_inp[MAXL], str[MAXL], info[MAXL];
    int i, n, j, j1;    
    FILE  *mmcif=NULL, *fadit_inp_in=NULL, *fadit_inp_out=NULL;    
    FILE  *fadit_text_in=NULL, *fadit_text_out=NULL;

    
    mmcif=fopen("entity-poly.mmcif", "w");
    write_poly_infor(mmcif, 0);

    get_template_files(adit_text, adit_inp);
    
    if((fadit_inp_in = fopen(adit_inp,"r"))==NULL){ //the log_script
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_inp);
        pinfo(info,0);
    }
        
    if (fadit_inp_in){
        fadit_inp_out=fopen("log_script.inp", "w");
        while(fgets(str, sizeof str, fadit_inp_in)!=NULL){
            fprintf(fadit_inp_out, "%s", str);
        }
        fclose(fadit_inp_out);
    }
   
    fadit_text_out=fopen("data_template.text", "w");
    if((fadit_text_in = fopen(adit_text,"r"))==NULL){ //the template
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_text);
        pinfo(info,0);
    }else{
        write_entity_poly_text(fadit_text_in, fadit_text_out);
    }

    for (i=0; i<npoly; i++){
        n=i+1;
        fprintf(mmcif," %d ", n);
        fprintf(fadit_text_out,"<molecule_entity_id= %d >\n",n);
        fprintf(fcgi,"ENTITY_ID = %d :",n);
      
        fprintf(mmcif, " '%s' ", poly_type[i]);            
        fprintf(fadit_text_out, "<molecule_entity_type=%s >\n",  poly_type[i]);   
        fprintf(fcgi, " ENTITY_TYPE=%s  :", poly_type[i]);
        

        fprintf(mmcif,"\n;");            
        fprintf(fadit_text_out, "<molecule_one_letter_sequence= \n");
        fprintf(fcgi, " ONE_LETTER_SEQUNCE= ");

        n=strlen(seq_match[i]);
        j1=0;
        for (j=0; j<n; j++){
            j1++;
            fprintf(mmcif, "%c", seq_match[i][j]);
            fprintf(fadit_text_out,"%c",seq_match[i][j]);
            fprintf(fcgi,"%c" ,seq_match[i][j]);
            
            if((j1)%70==0 && j<n-6 && seq_match[i][j]!=')' && seq_match[i][j+1]!=')' &&
               seq_match[i][j+2]!=')' && seq_match[i][j+3]!=')' && seq_match[i][j+4]!=')') {
                fprintf(mmcif,"\n");
                fprintf(fadit_text_out,"\n");
            } 
        }
 
        fprintf(mmcif,"\n;\n");
        fprintf(fadit_text_out," >\n");
        fprintf(fcgi," :");
        
        
        fprintf(mmcif, " %s ? ? \n",ch_match[i]); //new
        fprintf(fadit_text_out, "< molecule_chain_id= %s >\n",ch_match[i]);
        fprintf(fcgi, " CHAIN_ID=%s : ", ch_match[i]);

        write_poly_infor(fadit_text_out, 1);
        write_poly_infor(fcgi, 3);

    }
    
// extra button for CGI interface
    
    write_poly_infor(fadit_text_out, 2);
    write_poly_infor(fcgi, 4);

    while(fadit_text_in && fgets(str, sizeof str, fadit_text_in)!=NULL){
        fprintf(fadit_text_out, "%s", str);
    }
    
    fclose(fadit_text_out);
    fclose(mmcif);
  //  write_matth_coeff_to_data_template((char *) "data_template.text");
  
    return;  
}
  
void write_poly(int nnpoly, char **poly_type, int *best_nseq, char **matched_ch,
                char** atomname, char **resname, char *bseq, int **pptype, int **ch_idx,
                int **res_idx, int **ch_break)
/*Get entity, and generate many files.
  nch: number of chains;  ptype: 0 for RNA/DNA, 1 for protein; ch_idx: chain index
  bseq: 3->1 letter residue; chain: 2 letter chainID (for each residue)
*/
{
    char adit_text[MAXL],adit_inp[MAXL], str[MAXL], info[MAXL];
    int i, n,k, j1, npoly=0,mark=0;    
    FILE  *mmcif=NULL, *fadit_inp_in=NULL, *fadit_inp_out=NULL;    
    FILE  *fadit_text_in=NULL, *fadit_text_out=NULL;

    
    
    mmcif=fopen("entity-poly.mmcif", "w");
    write_poly_infor(mmcif, 0);
    
    get_template_files(adit_text, adit_inp);
    
    if((fadit_inp_in = fopen(adit_inp,"r"))==NULL){ //the log_script
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_inp);
        pinfo(info,0);
    }else{
        fadit_inp_out=fopen("log_script.inp", "w");
        while(fgets(str, sizeof str, fadit_inp_in)!=NULL){
            fprintf(fadit_inp_out, "%s", str);
        }
        fclose(fadit_inp_out);
    }
    
    fadit_text_out=fopen("data_template.text", "w");
    if((fadit_text_in = fopen(adit_text,"r"))==NULL){ //the template
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_text);
        pinfo(info,0);
    }else{
        write_entity_poly_text(fadit_text_in, fadit_text_out);
    }

    for (i=0; i<nnpoly; i++){
       
        n=best_nseq[i];
        if (ch_idx[n][1] - ch_idx[n][0] <2) continue;
        
        npoly++;
        fprintf(mmcif," %d ",npoly); //write to "entity-poly.mmcif" file
        fprintf(fadit_text_out,"<molecule_entity_id=%d >\n",npoly); //write to "data_template.cif" file
        fprintf(fcgi,"ENTITY_ID = %d :",npoly); //write to "cgi_value" file
      
        fprintf(mmcif, " '%s' ", poly_type[n]);            
        fprintf(fadit_text_out, "<molecule_entity_type=%s >\n",  poly_type[n]);   
        fprintf(fcgi, " ENTITY_TYPE=%s  :", poly_type[n]);
        

        fprintf(mmcif,"\n;");            
        fprintf(fadit_text_out, "<molecule_one_letter_sequence= \n");
        fprintf(fcgi, " ONE_LETTER_SEQUNCE= ");
 
        j1=0;
        for (k=ch_idx[n][0]; k<ch_idx[n][1]; k++){
            
            if(bseq[k]=='X' || bseq[k]=='?'){
                sprintf(str,"(%s)", resname[k]);
            }else {
                if (pptype[n][k]==1){
                    sprintf(str,"(D%c)", bseq[k]);
                }else if (pptype[n][k]==4){ //hybrid
                    if (find_atom(" O2'", atomname, res_idx[k][0], res_idx[k][1])>=0 ||
                        find_atom(" O2*", atomname, res_idx[k][0], res_idx[k][1])>=0) {
                        sprintf(str,"%c", bseq[k]);
                    }else{
                        sprintf(str,"(D%c)", bseq[k]);
                    }
                    
                } else{
                    sprintf(str,"%c", bseq[k]);
                }
            }
            if (ch_break[n][k]==0) {
                strcat(str,"????");
                mark++;
                
            }
	    strcpy(str, ""); //chenghua-20190325, str stores sequence, make it empty, force author to input
            fprintf(mmcif, "%s", str);
            fprintf(fadit_text_out,"%s",str);
            fprintf(fcgi,"%s" ,str);
            j1++;
            if((j1)%70==0) {
                fprintf(mmcif,"\n");
                fprintf(fadit_text_out,"\n");
            } 
        }
        fprintf(mmcif,"\n;\n");
        fprintf(fadit_text_out," >\n");
        fprintf(fcgi," :");
        
        
        fprintf(mmcif, " %s  ? ?\n",matched_ch[i]); //new
        fprintf(fadit_text_out, "< molecule_chain_id=%s >\n",matched_ch[i]);
        fprintf(fcgi, " CHAIN_ID=%s : ",matched_ch[i]);
        
        write_poly_infor(fadit_text_out, 1);
        write_poly_infor(fcgi, 3);
    }
    if (mark>0){                
        sprintf(info, "Warning: question marks are inserted in the sequence to remind you of the missing residues.\n");
        pinfo(info,1);
        sprintf(info, "Please replace the question marks by the missing residues in the sequence.\n");
        pinfo(info,1);
    }

    
// extra button for CGI interface
    
    write_poly_infor(fadit_text_out, 2);
    write_poly_infor(fcgi, 4);
    
    while(fadit_text_in && fgets(str, sizeof str, fadit_text_in)!=NULL){
        fprintf(fadit_text_out, "%s", str);
    }
    
    fclose(fadit_text_out);
    fclose(mmcif);

  //  write_matth_coeff_to_data_template((char *) "data_template.text");
  
    return;  
}
  

void get_entity(int nch, int *ptype, int **ch_idx, 
                char **resname, char *bseq, char **chain)
/*Get entity, and generate many files.
  nch: number of chains;  ptype: 0 for RNA/DNA, 1 for protein; ch_idx: chain index
  bseq: 3->1 letter residue; chain: 2 letter chainID (for each residue)
*/
{
    char chain_nam1[4], chain_nam2[4], info[200], entity_name[80];
    char adit_text[MAXL],adit_inp[MAXL], str[MAXL];
    char **chain_res,  *matched, *matched_comp, tmp[2560];
    int i,j, k, n,m,  *matched_num, *chlen;
    long **seq1_match,**seq2_match, nmatch, npoly,  j1;
    double percent=0;
    
    FILE  *mmcif=NULL, *fadit_inp_in=NULL, *fadit_inp_out=NULL;    
    FILE  *fadit_text_in=NULL, *fadit_text_out=NULL;

    chlen=new int[nch+1];

    
    remove("data_template.text");
    remove("log_script.inp");
    remove("entity-poly.mmcif");
    
    mmcif=fopen("entity-poly.mmcif", "w");
    write_poly_infor(mmcif, 0);

    get_template_files(adit_text, adit_inp);
    
    if((fadit_inp_in = fopen(adit_inp,"r"))==NULL){ //the log_script
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_inp);
        pinfo(info,0);
    }else{
        fadit_inp_out=fopen("log_script.inp", "w");
        while(fgets(str, sizeof str, fadit_inp_in)!=NULL){
            fprintf(fadit_inp_out, "%s", str);
        }
        fclose(fadit_inp_out);
    }

    
    fadit_text_out=fopen("data_template.text", "w");
    if((fadit_text_in = fopen(adit_text,"r"))==NULL){ //the template
        sprintf(info, "WARNING! Can not open the file (%s).\n",adit_text);
        pinfo(info,0);
    }else{
        write_entity_poly_text(fadit_text_in, fadit_text_out);
    }

    
    chain_res=new char *[nch+2]; //residue held in the chain.
    for (i=0; i<nch; i++){
        n=ch_idx[i][1]-ch_idx[i][0];
        chain_res[i]=new char [n+2]; 
        j=0;
        for (k=ch_idx[i][0]; k<=ch_idx[i][1]; k++){
            chain_res[i][j]=bseq[k];
            j++;
        }
        chain_res[i][j] = '\0';
//        printf("i,ch, seq= %d %s %d %d %s\n", i, chain[ch_idx[i][0]],ch_idx[i][0], ch_idx[i][1], chain_res[i]);
    }
    
    
    seq1_match = lmatrix(0,50, 0,40);
    seq2_match = lmatrix(0,50, 0,40);
    
    npoly=0;
    matched = new char [nch*40];
    matched_comp = new char [nch*40];
    strcpy(matched,"");
    strcpy(matched_comp,"");
    matched_num=new int [nch*40];
    for (k=0; k<nch+4; k++) matched_num[k]=-1;
    
    for (i=0; i<nch; i++){
//        if(ptype[i]== -1)continue; /*rid of ligand*/
        strcpy(chain_nam1, chain[ch_idx[i][0]]);
        matched_num[i]=i;
        
        if (strstr(matched_comp, chain_nam1)) continue; /* matched with this chain*/
        chlen[i]=ch_idx[i][1] - ch_idx[i][0] +1;
//        printf("cha1=%s, %d  %d %d  mach=%s %d\n", chain_nam1, i,ch_idx[i][0], ch_idx[i][1],matched_comp,chlen[i]);

        k=0;
        for (j=i+1; j<nch; j++){
            if(ptype[i] != ptype[j]) continue;
            strcpy(chain_nam2, chain[ch_idx[j][0]]);
//            printf("seq1=%d  %d  %s | %s\n", i, j, chain_res[i], chain_res[j]);
            
            seq_alignment(i, j, chain_res, &percent, &nmatch, seq1_match, seq2_match);
//      fasta_seq_align(i, j, chain_res, &score);
            
            if((((ptype[i]==0 && ptype[j] ==0) || (ptype[i]==1 && ptype[j] ==1)) && percent>80.0 )|| //Nucleic acids 
               (ptype[i]==2 && ptype[j] ==2 && percent>85.0 )) { //protein
                strcat(matched, chain_nam2);
                strcat(matched,",");
                strcat(matched_comp, chain_nam2);
                strcat(matched_comp,",");
                matched_num[j]=j;
                chlen[j]=ch_idx[j][1] - ch_idx[j][0] +1;
            }
//            printf("Sequence similarity(%s:%s)=%.1f%%; nmatch=%2ld\n", chain_nam1, chain_nam2, percent, nmatch);                    
        }
        printf("Chain (%s) is matched with chain (%s) (i=%d).\n",chain_nam1, matched, i);
        n=0;
        m=i;
        for (k=0; k<nch; k++){ //get the longest length of the chain.
            if (matched_num[k]>=0) {
                printf ("matched %4d %4d\n",matched_num[k], chlen[k]);
                if (chlen[k]>n){
                    n =  chlen[k];
                    m = matched_num[k];
                }
            }
        }
        printf("the best chain = %d %d %d %d\n", m, chlen[m], ch_idx[m][0], ch_idx[m][1]);
        
        for (k=0; k<nch; k++) matched_num[k]=-1;
        
        npoly++;
        fprintf(mmcif," %ld ",npoly);
        fprintf(fadit_text_out,"<molecule_entity_id=%ld >\n",npoly);
        fprintf(fcgi,"ENTITY_ID = %ld :",npoly);
      
        type_of_entity(ptype, i, ch_idx, bseq, entity_name);
        fprintf(mmcif, " %s ", entity_name);            
        fprintf(fadit_text_out, "<molecule_entity_type=%s >\n", entity_name);   
        fprintf(fcgi, " ENTITY_TYPE=%s  :",entity_name);
        
        fprintf(mmcif,"\n;");            
        fprintf(fadit_text_out, "<molecule_one_letter_sequence= \n");
        fprintf(fcgi, " ONE_LETTER_SEQUNCE= ");

        
        j1=0;
        for (k=ch_idx[i][0]; k<=ch_idx[i][1]; k++){
                
            
            if(bseq[k]=='X' || bseq[k]=='?'){
                fprintf(mmcif,"(%s)", resname[k]);            
                fprintf(fadit_text_out,"(%s)",resname[k] );
                fprintf(fcgi,"(%s)",resname[k]);
            }else{
                if (ptype[i]==1){ //DNA
                    fprintf(mmcif, "(D%c)", bseq[k]);
                    fprintf(fadit_text_out,"(D%c)",bseq[k]);
                    fprintf(fcgi,"(D%c)",bseq[k]);
                }else{
                    fprintf(mmcif, "%c", bseq[k]);
                    fprintf(fadit_text_out,"%c",bseq[k]);
                    fprintf(fcgi,"%c",bseq[k]);
                }
            }
            
            j1++;
            if((j1)%70==0) {
                fprintf(mmcif,"\n");
                fprintf(fadit_text_out,"\n");
            }
            
        }

        fprintf(mmcif,"\n;\n");
        fprintf(fadit_text_out," >\n");
        fprintf(fcgi," :");
        
        sprintf(tmp, "%s,%s",chain_nam1,matched);
        rid_of_space(tmp);
        strcpy(matched, ""); //repeat
        
        fprintf(mmcif, " %s\n",tmp);
        fprintf(fadit_text_out, "< molecule_chain_id=%s >\n",tmp);
        fprintf(fcgi, " CHAIN_ID=%s : ",tmp);

        write_poly_infor(fadit_text_out, 1);
        write_poly_infor(fcgi, 3);
    }
    
// extra button for CGI interface
    write_poly_infor(fadit_text_out, 2);
    write_poly_infor(fcgi, 4);
    

    while(fadit_text_in && fgets(str, sizeof str, fadit_text_in)!=NULL){
        fprintf(fadit_text_out, "%s", str);
    }
    
    
    fclose(fadit_text_out);
    fclose(mmcif);
//    write_matth_coeff_to_data_template((char *) "data_template.text");
  
    
    return;
    
}

void write_matth_coeff_to_data_template(char *inpfile)
// update matth_coeff and solvent constant by extract -sol data_template.text
{
    char  value[50000],seq[50000],  item[60], str[100];
    char seq_type[60], chain_id[512];
    char sg_name_H_M[20],sg_name_table[20];
    int  key=0,  n1=0, n2=0, n3=0, nsym;
    float mass=0,mass_mol=0, cell[7], cell_volume=0, matthew_coeff=0, solvent_const=0;
    FILE  *fp=NULL, *ftmp;

        
    if((fp=fopen(inpfile, "r"))==NULL){
        return;
    }
    
    ftmp = tmpfile();
    while(fgets(str, sizeof str, fp) !=NULL){
        fprintf(ftmp, "%s", str);
    }
    
    rewind(ftmp);
    while(1) {
        values_in_bracket(ftmp,  item, value,  &key);
        if(key>0)break;
        
        if(!strcmp_case(item, "space_group")){
            strcpy(sg_name_H_M, value);
            
        }else if(!strcmp_case(item, "unit_cell_a") ||
                 !strcmp_case(item, "a")){
            cell[0] = atof (value);
            
        }else if(!strcmp_case(item, "unit_cell_b")||
                 !strcmp_case(item, "b")){
            cell[1] = atof (value);
           
        }else if(!strcmp_case(item, "unit_cell_c")||
                 !strcmp_case(item, "c")){
            cell[2] = atof (value);
            
        }else if(!strcmp_case(item, "unit_cell_alpha")||
                 !strcmp_case(item, "alpha")){
            cell[3] = atof (value);
           
        }else if(!strcmp_case(item, "unit_cell_beta")||
                 !strcmp_case(item, "beta")){
            cell[4] = atof (value);
            
        }else if(!strcmp_case(item, "unit_cell_gamma")||
                 !strcmp_case(item, "gamma")){
            cell[5] = atof (value);
        }

        
        if(strstr_case(item, "entity_type")){
            strcpy(seq_type, value);
            n1=1;
        }else if(strstr_case(item, "one_letter")){
            strcpy(seq, value);
            n2=1;
        }else if(strstr_case(item, "chain_id")){
            strcpy(chain_id, value);
            n3=1;
        }
        if(n1>0 && n2>0 && n3>0 && strlen(seq)>0 && strlen(chain_id)>0){
            if(strstr_case(seq_type,"polypeptide")){
                mass = mass + residue_mass(seq, chain_id, (char *)"amino_acids");
            }else{
                mass = mass + residue_mass(seq, chain_id, (char *)"nucleic_acids");
            }
            n1=0; n2=0; n3=0;
        }
    }
    mass_mol=mass;
    
    cell_volume = get_cell_volume(cell);
    nsym = get_symmetry_num(sg_name_H_M, sg_name_table);
    mass = nsym*mass;
    
    if(cell_volume >1 && mass>0){
        matthew_coeff = cell_volume/mass;
        solvent_const =(1.0-1.0/(matthew_coeff * 0.602*1.34))*100.0;
    }

    printf("\nSPG=%s (%d); matt=%.2f, solvent=%.2f, mass=%.1f, cell_vol=%.1f\n",
           sg_name_H_M, nsym,matthew_coeff,solvent_const, mass_mol, cell_volume);

    if(matthew_coeff<10 && matthew_coeff>0.5 && solvent_const<95 && solvent_const>5){
        fp=fopen("data_template.text", "w");
        rewind(ftmp);
        while(fgets(str, sizeof str, ftmp) !=NULL){
            
            if(pattern5(str, "<", "crystals_solvent_content", "=", ">" , "")){
                fprintf(fp, "<crystals_solvent_content = \"%.1f \">\n",solvent_const);
            }else if (pattern5(str, "<", "crystals_matthews_coefficient", "=", ">" , "")){
                fprintf(fp, "<crystals_matthews_coefficient = \"%.1f \">\n",matthew_coeff);
            }else{
                fprintf(fp,"%s", str);
            }
        }
    }
    fclose(ftmp);
}

float get_cell_volume(float *cell)
{
    float  alpha, beta,gamma ,cell_volume;
    
    alpha = 3.14159 * cell[3]/180;
    beta  = 3.14159 * cell[4]/180;
    gamma = 3.14159 * cell[5]/180;
    
    cell_volume = cell[0] * cell[1] * cell[2] *
        sqrt(1.- cos(alpha)*cos(alpha) - cos(beta)*cos(beta) - cos(gamma)*cos(gamma)
             + 2.0 * cos(alpha) * cos(beta) * cos(gamma));
    return cell_volume;
}

float residue_mass(char *seq, char *chain_id, char *residue_type)
{
    int i, j, k, nres=0, n_chain=1, n_modify=0, len_id, len_seq,nb=0;
    float mass = 0, mass_tot=0;
    
    
    len_id = strlen(chain_id);
    len_seq = strlen(seq);
    if(len_id<=0 || len_seq<=0) return 0;

    for(i=0; i< len_id; i++){
        if(chain_id[i] == ',')n_chain++;
    }
    
    for(i=0; i< len_seq; i++){
        if(seq[i] == '?' ){
            nb++;
            continue;
        }
        
        if(seq[i] == ' ' || seq[i] == '\n')continue;
        if(seq[i] == '('){
            j=i;
            k=0;
            for(i=j; i< len_seq; i++){
                if(seq[i] == ' ' || seq[i] == '\n')continue;
                k++;
                if(seq[i] == ')'){
                    n_modify++;
                    if(k>5)printf("Warning! Characters (=%d) in () > 3\n", k);
                    
                    break;
                }
            }
            if(k>0) continue;      
        }
        nres++;
//        printf("%c", seq[i]);
        if(!strcmp_case(residue_type, "nucleic_acids")){
            mass = mass + single_residue_mass((char *)"nucleic_acids", seq[i]);
        }else if (!strcmp_case(residue_type, "amino_acids")){
            mass = mass + single_residue_mass((char *)"amino_acids", seq[i]);
        } 
    }
    
//    printf("\n modify= res=%d %d  %d  %.2f\n", n_chain, n_modify, nres, mass );

    
    
    if(!strcmp_case(residue_type, "nucleic_acids")){
        mass_tot = (mass + n_modify*300)* n_chain;
        return mass_tot ;
    } else if (!strcmp_case(residue_type, "amino_acids")){
        mass_tot = (mass + n_modify*110)* n_chain;
        return mass_tot ;
    } else{
        mass_tot = (mass + n_modify*110)* n_chain;
        return mass_tot ;
    }
    
    
}

float single_residue_mass(char *residue_type, char residue_name)
{
    int i;

    if(strstr_case(residue_type, "nucleic_acids")) {
        for (i = 0; i < NUM_MASS_TABLE_NUCLEIC_ACIDS; i++) {
            if (mass_table_nucleic_acids[i].Residue_1[0] == residue_name){
                return mass_table_nucleic_acids[i].Residue_Mass_H;
            }
        }
        return 290;
    }else if(strstr_case(residue_type, "amino_acids")) {
        for (i = 0; i < NUM_MASS_TABLE_AMINO_ACIDS; i++) {
            if (mass_table_amino_acids[i].Residue_1[0] == residue_name){
                return mass_table_amino_acids[i].Residue_Mass_H;
            }
        }
        return 110;
           
    }else{
        return 110;
    }
}

int get_symmetry_num(char *sg_name_inp, char *sg_name_out)
{
       int i, n=0;
       char sg_name_inp_tmp[20],sg_name_table_tmp[20];

       strcpy(sg_name_inp_tmp, sg_name_inp);
       rid_of_space(sg_name_inp_tmp);
           
       for (i = 0; i < NUM_SPACE_GROUP; i++) {
           strcpy(sg_name_table_tmp, space_group_lib[i].space_group_name);
           rid_of_space(sg_name_table_tmp);
           
           if (!strcmp_case(sg_name_inp_tmp, sg_name_table_tmp)){
               strcpy(sg_name_out, space_group_lib[i].space_group_name);
               n = space_group_lib[i].nsym;
               break;
           }
       }
       if(n==0){ //space group name wrong!
           strcpy(sg_name_out, sg_name_inp);
           n=1;
       }
       return n;
}


void write_entity_poly_text(FILE *fadit_in,  FILE *fadit_out)
//write template upto cases of residues ...
//Never change Enter Crysta.. and cases of residues modele ..
{
    char str[512];


    if(!strcmp_case(METHOD, "NMR")){
        while(fgets(str, sizeof str, fadit_in)!=NULL){
            fprintf(fadit_out, "%s", str);
            if(strstr_space(str, "Please manually add the sequence")){
                fprintf(fadit_out,"\n");
                break;
            } 
        }
    }else{ //Xray
        while(fgets(str, sizeof str, fadit_in)!=NULL){
            fprintf(fadit_out, "%s", str);
            if(strstr_space(str, "Enter Crystallographic Data")) {
                
                fprintf(fadit_out,"\n<space_group = %s> (use International Table conventions)\n",
                        cell.space_group);
                fprintf(fadit_out,"<unit_cell_a     = %s >\n", cell.a);
                fprintf(fadit_out,"<unit_cell_b     = %s >\n", cell.b);
                fprintf(fadit_out,"<unit_cell_c     = %s >\n", cell.c);
                fprintf(fadit_out,"<unit_cell_alpha = %s >\n", cell.alpha);
                fprintf(fadit_out,"<unit_cell_beta  = %s >\n", cell.beta);
                fprintf(fadit_out,"<unit_cell_gamma = %s >\n", cell.gamma);

                break;
            }
        }
        
        while(fgets(str, sizeof str, fadit_in)!=NULL){
            fprintf(fadit_out, "%s", str);
            
            if(strstr_space(str, "cases of residues modeled as Ala or Gly due to missing side-chain density")){
                fprintf(fadit_out,"\n");
                break;
            } 
        }
        
    }
    
    
}

void read_pdb_head(char *pdbfile)
{
    char str[100], sym[20];
    FILE *fp;
	
    if((fp = fopen(pdbfile, "r"))==NULL){
        open_file_error(pdbfile, "read_pdb_head");
    }
    
    strcpy(cell.space_group, "?");
    strcpy(cell.a, "?");
    strcpy(cell.b, "?");
    strcpy(cell.c, "?");
    strcpy(cell.alpha, "?");
    strcpy(cell.beta, "?");
    strcpy(cell.gamma, "?");
    cell.mol_id=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if (!strncmp(str, "COMPND    MOL_ID:", 16) && strchr(str, ';')){
            cell.mol_id = cell.mol_id + 1;
        }else if (!strncmp_case(str, "CRYST1", 6)){


            strncpy(cell.a,str+7, 9);
            cell.a[10]='\0';
            
            strncpy(cell.b,str+16, 9);
            cell.b[10]='\0';
            
            strncpy(cell.c,str+25, 9);
            cell.c[10]='\0';
            
            strncpy(cell.alpha,str+34, 7);
            cell.alpha[8]='\0';
            
            strncpy(cell.beta,str+41, 7);
            cell.beta[8]='\0';
            
            strncpy(cell.gamma,str+48, 7);
            cell.gamma[8]='\0';
            
            get_string(str, 55, 66, sym);
            replace_char(sym, ':', ' ');
            space_group_convertor(sym,cell.space_group);
            
//            strncpy(cell.space_group,str+55, 10);
//            cell.space_group[11] = '\0';
            break;
        }else if (!strncmp(str, "ATOM", 4)){
            break;
        }else if(!strncmp_case(str, "REMARK ", 7) && pattern5(str, "sg=","a=", "b=","c=","alpha=") ){
            
                
            get_data_set_id(str,"sg=", "a=", sym);
            if(strchr(sym,'('))
                replace_char(sym,'(', ' ');
            if(strchr(sym,')'))
                replace_char(sym,')', ' ');
            
            space_group_convertor(sym,cell.space_group);

            get_one_string_after_id(str,"a=", cell.a);
            get_one_string_after_id(str,"b=", cell.b);
            get_one_string_after_id(str,"c=", cell.c);
            get_one_string_after_id(str,"alpha=", cell.alpha);
            get_one_string_after_id(str,"beta=", cell.beta);
            get_one_string_after_id(str,"gamma=", cell.gamma);
            
            break;
        }
            

    }
    fclose(fp);
 
}

void get_mmcif(char *inpfile)
/* parse the information from log_script.inp and use the pdb_extract program
   to get a mmcif file.
*/
{
    char value[5000]; /* value in " " of <> */
    char item[80];  /* characters for the first item in <> */
    
    char sf_dt[50],sf_prog[20], sf_file[MAXL];
    char ref_sf_prog[20],ref_sf_file[MAXL],log_sf_file[MAXL];
    char ref_sf_detail[2000], scale_sf_detail[2000], free_set[10];
    
    char scale_p[20], scale_log[MAXL],scale_cif[MAXL];
    char index_p[20],index_log[MAXL],index_cif[MAXL];
    char mr_e[10],mr_p[20],mr_log[MAXL];
    char phase_e[10],phase_p[20],phase_log[MAXL],phase_pdb[MAXL],phase_cif[MAXL];
    char dm_d[20], dm_log[MAXL], dm_pdb[MAXL], dm_cif[MAXL];
    char ref_r[20], ref_log[MAXL], ref_pdb[MAXL];
    
    char data_template[MAXL], data_template_tmp[MAXL];
    char tmp[2000], sf_output[MAXL], stat_output[MAXL];
    char *temp, pdb_extract[2000], pdb_extract_sf[2000];
    int  entity = 0, n=0, key=0;
    FILE *finp, *fadit;


    mr_p[0] = '\0';
    mr_log[0] = '\0';
    data_template_tmp[0] = '\0';

    if((finp=fopen(inpfile,"r"))==NULL){
        open_file_error(inpfile, "get_mmcif");
    }
    
    (!strcmp_case(CCP4,"yes"))?temp=getenv("CCP4"):temp=getenv("PDB_EXTRACT");
    if(temp != NULL) {
        strcpy(pdb_extract, "");
        catstring(pdb_extract, temp, "/bin/pdb_extract ",  " ");
        strcpy(pdb_extract_sf, "");
        catstring(pdb_extract_sf, temp, "/bin/pdb_extract_sf ",  " ");
//        catstring(pdb_extract_sf, temp, "/bin/sf_convert ",  " ");
    }else{
        if(!strcmp_case(CCP4,"yes"))
            printf("Wrong! CCP4 environment was not set.\n");
        else
            printf("Wrong! PDB_EXTRACT environment was not set.\n");
        printf("Please look at the README instruction in the package.\n");
        return;
    }

    
    strcpy(phase_cif,""); strcpy(phase_pdb,""); strcpy(phase_log,"");
    strcpy(dm_cif,""); strcpy(dm_pdb,""); strcpy(dm_log,"");
    strcpy(ref_pdb,"");  strcpy(ref_log,"");
    strcpy(scale_cif,""); strcpy(scale_log,"");strcpy(log_sf_file,"");
    strcpy(index_cif,""); strcpy(index_log,"");
    
    strcpy(sf_output,"pdb_extract_sf.mmcif"); /*output default*/
    strcpy(stat_output,"pdb_extract.mmcif");  /*output default*/
    
    strcpy(ref_sf_prog, "");
    strcpy(ref_sf_file, "");
    strcpy(sf_dt, "");
    strcpy(sf_prog, "");
    strcpy(sf_file, "");
    strcpy(ref_sf_detail, "");
    strcpy(scale_sf_detail, "");
    strcpy(free_set, " 0 ");
    
    while(1) { /* parsing values  in < >*/
	value[0] = '\0';
        values_in_bracket(finp,  item, value,  &key);
//        printf("item,value=|%s|%s|\n", item, value);
        
        if(key>0) break;
        if(strlen(value)<=0) continue;
        
        if(!strcmp_case(item,"scale_data_type")||
           !strcmp_case(item,"reflection_data_type")){
            catstring(pdb_extract_sf, " -sf_type ", value, " ");
            
        }else if(!strcmp_case(item,"program_name")){
            catstring(pdb_extract_sf, " -i ", value, " ");
            
        }else if(!strcmp_case(item,"reflection_data_free_set")){
            catstring(pdb_extract_sf, " -freer ", value, " ");
            
        }else if(!strcmp_case(item,"reflection_data_file_name")){
//            catstring(pdb_extract_sf, " -idat ", value, " ");
            catstring(pdb_extract_sf, " -sf ", value, " ");
            
        }else if(!strcmp_case(item,"reflection_data_detail")){
            sprintf(tmp," -detail \"%s\" ", value);
            strcat(pdb_extract_sf, tmp);

            
            
        }else if(strstr_case(item,"data_indexing_software")){/*Data Indexing */
            strcpy(index_p, value);
        }else if(strstr_case(item,"data_indexing_LOG_file_name")){
            strcpy(index_log, value);
        }else if(strstr_case(item,"data_indexing_CIF_file_name")){
            strcpy(index_cif, value);

        }else if(strstr_case(item,"data_scaling_software")){/*Data Scaling */
            strcpy(scale_p, value);
        }else if(strstr_case(item,"data_scaling_LOG_file_name")){
            strcpy(scale_log, value);
        }else if(strstr_case(item,"data_scaling_CIF_file_name")){
            strcpy(scale_cif, value);


        }else if(!strncmp_case(item,"mr_software",10)){  /*MR*/
            strcpy(mr_e, "MR");
            strcpy(mr_p, value);
        }else if(!strncmp_case(item,"mr_log_file_log_",14)){
            catstring(mr_log, " ", value, " ");
        }else if(!strncmp_case(item,"phasing_method",12)){  /*phasing*/
            strcpy(phase_e, value);
        }else if(!strncmp_case(item,"phasing_software",12)){
            strcpy(phase_p, value);
        }else if(!strncmp_case(item,"phasing_log_file_cif_",21)){
            catstring(phase_cif, " ", value, " ");
        }else if(!strncmp_case(item,"phasing_log_file_pdb_",21)){
            catstring(phase_pdb, " ", value, " ");
        }else if(!strncmp_case(item,"phasing_log_file_log_",21)){
            catstring(phase_log, " ", value, " ");
            
        }else if(!strncmp_case(item,"dm_software",10)){  /*dm*/
            strcpy(dm_d, value);
        }else if(!strncmp_case(item,"dm_log_file_cif_",16)){
            catstring(dm_cif, " ", value, " ");
        }else if(!strncmp_case(item,"dm_log_file_pdb_",16)){
            catstring(dm_pdb, " ", value, " ");
        }else if(!strncmp_case(item,"dm_log_file_log_",16)){
            catstring(dm_log, " ", value, " ");
            
        }else if(!strcmp(item,"refine_software")){ /*refinement*/
            strcpy(ref_r, value);
            
        }else if(!strcmp_case(item,"reflection_data_detail")){
            if (strlen(value)>0) sprintf(ref_sf_detail, " -detail %s ", value);
            
        }else if(!strcmp_case(item,"reflection_data_free_set")){ 
            strcpy(free_set, value);
            
        }else if(!strcmp_case(item,"scale_data_detail")){ 
            if (strlen(value)>0) sprintf(scale_sf_detail, " -detail %s ", value);
            
        }else if(!strncmp_case(item,"refine_log_file_cif_",20) ||
                 !strncmp_case(item,"refine_xyz_file_cif_",20) ||
                 !strncmp_case(item,"refine_log_file_pdb_",20) ||
                 !strncmp_case(item,"refine_xyz_file_pdb_",20) ||
                 !strcmp_case(item,"refine_xyz_file") ){
            
            strcpy(ref_pdb, value);
            
        }else if(!strncmp_case(item,"refine_log_file_log_",20) ||
                 !strcmp_case(item,"refine_xyz_file") ){
            strcpy(ref_log, value);

            
        }else if(!strcmp_case(item,"data_template_file") ){  /*Data Template File*/
            strcpy(data_template, value);

        }else if(!strcmp(item,"sf_output") ){  /*output file*/
            strcpy(sf_output, value);
        }else if(!strcmp(item,"statistics_output") ){
            strcpy(stat_output, value);
            
        }
        
    }
    
/* --------------------------Use pdb_extract command------------------------*/
    
// extract scale statistics for indexing
    if (strlen(index_p)>0 &&
        (strlen(index_log)>0 || strlen(index_cif) ) ){
        catstring(pdb_extract, " -i ", index_p, " ");

        if(strlen(index_log)>0){
            catstring(pdb_extract, " -ilog ", index_log, " ");
        }
        
        if(strlen(index_cif)>0){
            catstring(pdb_extract, " -icif ", index_cif, " ");
       }
    }

    
// extract scale statistics for scaling
    if (strlen(scale_p)>0 &&
        (strlen(scale_log)>0 || strlen(scale_cif) ) ){

        catstring(pdb_extract, " -s ", scale_p, " " );

        if(strlen(scale_log)>0){
            catstring(pdb_extract, " -ilog ", scale_log, " ");
        }
        if(strlen(scale_cif)>0){
            catstring(pdb_extract, " -icif ", scale_cif, " ");
        }
    }

// extract scale statistics for phasing
    if (strlen(log_sf_file)>0 && strlen(sf_prog)>0){ 
        sprintf(tmp, " -sp  %s  -ilog %s ", sf_prog, log_sf_file);
        strcat(pdb_extract, tmp);
    }

/* extract MR statistics*/
    if (strlen(mr_p)>0 && strlen(mr_log)>0 ){  
        sprintf(tmp, " -e  MR  -m   %s  -ilog  %s ", mr_p, mr_log);
        strcat(pdb_extract, tmp);
    } 
 
/* extract phasing statis*/
    if(strlen(phase_e)>0 && strlen(phase_p)>0 && 
       (strlen(phase_cif)>0 || strlen(phase_pdb)>0 || strlen(phase_log)>0)){
        sprintf(tmp, " -e  %s  -p   %s  ", phase_e , phase_p);
        strcat(pdb_extract, tmp);
          
        if(strlen(phase_cif)>0){
            catstring(pdb_extract, " -icif ", phase_cif, " ");
        }
        
        
        if(strlen(phase_log)>0){
            catstring(pdb_extract, " -ilog ", phase_log, " ");
        }
        
        if(strlen(phase_pdb)>0){
            catstring(pdb_extract, " -ipdb ", phase_pdb, " ");
        }
        
    }
   
/* extract dm statis*/
    if(strlen(dm_d)>0  &&  
       (strlen(dm_cif)>0 || strlen(dm_pdb)>0 || strlen(dm_log)>0)){
        catstring(pdb_extract, " -d ", dm_d, " ");

        if(strlen(dm_cif)>0){
            catstring(pdb_extract, " -icif ", dm_cif, " ");
        }
        
        if(strlen(dm_pdb)>0){
            catstring(pdb_extract, " -ipdb ", dm_pdb, " ");
        }
        
        if(strlen(dm_log)>0){
            catstring(pdb_extract, " -ilog ", dm_log," ");
        }
    }
    
/* extract refine statis*/        
    if(strlen(ref_r)>0 && strlen(ref_pdb)>0 ){        
        catstring(pdb_extract, " -r ", ref_r, " ");
        
        (is_cif(ref_pdb)>1)? strcat(pdb_extract, " -icif "):strcat(pdb_extract, " -ipdb ");
        strcat(pdb_extract, ref_pdb);
        
        if(strlen(ref_log)>0){
            (is_cif(ref_log)>1)? strcat(pdb_extract, " -icif "):strcat(pdb_extract, " -ilog ");
            strcat(pdb_extract, ref_log);
        }   
    }

/* extract data template file*/        
    if(strlen(data_template)>0){
        catstring(pdb_extract, " -ient ", data_template, " ");
        
    }else { //test if the log_script.inp has entity_poly information
        
        rewind(finp);
        while(1) {
            values_in_bracket(finp,  item, value,  &key);
            if(key>0)break;
            if( strlen(value)<=0 ) continue;
            rid_of_front_end_space(value);
            if(!strcmp_case(item,"ONE_LETTER_SEQUNCE") && strlen(value)>0){  
                entity=1;
                break;
            }
        }

        if(entity==1){
            sprintf(data_template_tmp, "adit_text_%d", getpid());
            fadit =  fopen(data_template_tmp, "w"); 
            parse_template_to_mmcif(fadit, inpfile,1); 
            fclose (fadit);
            catstring(pdb_extract, " -ient ", data_template_tmp, " ");
        }
    }
    
    catstring(pdb_extract, " -o ", stat_output, " ");

    printf("\n============Structure Solution Statistics===============\n");
    n=system(pdb_extract); /* phasing + dm + refinement stat */
    check_system_call(n,"phasing+dm_refine");
//    printf("command=%s\n", pdb_extract);
    if( strlen(data_template_tmp)>1) delete_file(data_template_tmp);
    fclose(finp);


/* ------------------------Use pdb_extract_sf command-----------------------*/
    if (strstr_case(pdb_extract_sf, "-idat") ||  strstr_case(pdb_extract_sf, "-sf")){
        
        catstring(pdb_extract_sf, " -pdb ", stat_output, "  ");        
        catstring(pdb_extract_sf, " -o  mmcif ", " ", "  ");        
        catstring(pdb_extract_sf, " -out ", sf_output, " >/dev/null ");
        
//    printf("command=%s\n", pdb_extract_sf);
        printf("\n=========Converting structure factors to mmCIF===========\n");
        n=system(pdb_extract_sf);
        check_system_call(n,"pdb_extract_sf");
        printf("\nThe output SF file = %s\n",sf_output);
        
    }

    remove("mtzdmp.log");
    remove("sf_format_guess.text");

    
}

void catstring(char *orig, const char *ss1, const char *ss2, const char *ss3)
{
    strcat (orig, ss1);
    strcat (orig, ss2);
    strcat (orig, ss3);
}

    
