/* CIF utilities added by Chenghua, most for cif2cif conversion */
// utilCIF.c created by Chenghua on 2020-10-29
// some of the early-created CIF functions imbeded in pdb_extract.C are moved here
#include <fstream>
#include <string> // use standard C++ lib instead of string.h from C lib
#include <iostream>
#include <algorithm> 

std::string ltrim(const std::string& s)
//trim left-side white spaces from a string, return trimmed string
{
  const std::string WHITESPACE = " \n\r\t\f\v";  
  size_t start = s.find_first_not_of(WHITESPACE);
  return (start == std::string::npos) ? "" : s.substr(start);
}

std::string getHeader(std::string line, unsigned int num_chars)
//get the first few non-white character of a line as header, return the header
{
  std::string header = "";
  std::string line_trimed = ltrim(line);
  if (line_trimed.length() > num_chars){
    header = line_trimed.substr(0,num_chars);
  }
  return header;
}

void  convert_cif2rcsbcif(char *inpfile, char *inpfile_new)
//cif2cif conversion, note that this is NOT ONLY for coordinates cif
{
  std::string filepath_in = inpfile; // convert C character array to C++ string by overload '=' operator
  std::string filepath_out = inpfile_new;

  int block_num = 0; // set block number, later track block number to only output the 1st data block
  std::string line; // variable for each line
    
  std::ifstream file_in; // input file handler
  file_in.open(filepath_in.c_str()); // must use c_str() for older version c++ compiler
  std::ofstream file_out; // outout file handler
  file_out.open(filepath_out.c_str());
    
  if (file_in.is_open()){
    while (getline(file_in,line)){ // go through each line
      if (getHeader(line, 5) == "data_"){ // match the first 5 non-white characters to "data_" of datablock header 
	block_num += 1;
      }
      if (block_num <= 1){  //must handle block_num=0 for reading cif template without "data_" header
	file_out << line << '\n'; // output the first block only
      }
    }
  }
    
  file_in.close();
  file_out.close();
}

int write2ndPlusDatablocks(std::string filepath, std::string filepath_out)
//for cif input file from "filepath", check if there are 2+ datablocks;
//if there are 2+ datablocks, split the data of 2nd and further data blocks into a new file "filepath_out";
//"filepath_out" will be used for later merged back;
//return number of datablocks
{
  std::string line;
  std::string header;
  std::ifstream file;
  file.open(filepath.c_str()); //must use .c_str() for older version c++ compiler

  std::ofstream file_out;
  file_out.open(filepath_out.c_str());

  int header_count = 0;
  if (file.is_open()){
    while (getline(file,line)){
      header = getHeader(line, 5);
      if (header == "data_"){
	header_count += 1;
      }
      if (header_count > 1){ //if there are 2+ datablocks, write 2nd and further to file_out
	file_out << line << '\n';
      }
    }
    file.close();
    file_out.close();
  }else{
    std::cout << "Unable to open " << filepath << "\n";
    exit(0);
  }
  return header_count;
}

int checkModelCif(int argc, char **argv)
//check if input model if in cif format, and if yes, go check number of datablocks
//return number of datablocks
{
  std::string filepath;
  std::string filepath_2nd = "cif2ndPlusDatablocks.cif";
  int num_datablocks = 0;
  std::string arg;
  for(int i=0; i<argc-1; i++){
    arg = argv[i];
    std::transform(arg.begin(), arg.end(), arg.begin(), ::toupper); // ::toupper in the global namespace comes from C
    if (arg == "-ICIF"){
      //if (!strcmp_case(argv[i],"-ICIF") ){
      filepath = argv[i+1];
      num_datablocks = write2ndPlusDatablocks(filepath, filepath_2nd);
    }
  }
  return num_datablocks;
}

void append2ndPlusDatablocks(std::string filepath_1st, std::string filepath_2ndPlus)
//at the end of cif file process of the 1st datablock, if there were 2+ datablocks, append the 2nd and further 
{
  std::ifstream file_1st(filepath_1st.c_str());
  std::ifstream file_2ndPlus(filepath_2ndPlus.c_str());
  std::ofstream file_combined_tmp("datablocks_combined.cif"); //create a temp combined file
  file_combined_tmp << file_1st.rdbuf() << file_2ndPlus.rdbuf();
  file_1st.close();
  file_2ndPlus.close();
  file_combined_tmp.close();

  std::ifstream file_combined_from("datablocks_combined.cif"); //copy back to the original fout name
  std::ofstream file_combined_to(filepath_1st.c_str());
  file_combined_to << file_combined_from.rdbuf();
}

int countCifItemInFile(std::string item, std::string filepath){
    std::string line;
    std::string header;
    std::ifstream file;
    file.open(filepath.c_str());

    int header_count = 0;
    if (file.is_open()){
      while (getline(file,line)){ //check how many times an item appears
	header = getHeader(line, item.length());
	if (header == item){
	  header_count += 1;
	}
      }
      file.close();
    }else{
      std::cout << "Unable to open " << filepath << "\n";
    }
    return header_count;
}

void addEmptySymmetry(std::string filepath){
  std::fstream file;
  file.open(filepath.c_str(), std::fstream::in | std::fstream::out | std::fstream::app);
  file << "#\n";
  file << "_symmetry.entry_id                         ?\n";
  file << "_symmetry.space_group_name_H-M             ?\n";
  file << "#\n";
  file.close();
}

int countReflnsBwilson(std::string filepath)
//for special situation where model file has "_reflns.B_iso_Wilson_estimate", e.g. Phenix cif
//and user use -r/-i option to get "reflns" data from log file;
//check the output file from write_mmcif that could have duplicate "reflns" categories;
//if there are 2 "reflns", remove the 1st one that comes from model, use log file parsed data
//return number of "_reflns.B_iso_Wilson_estimate"
{
  std::string line;
  std::string header;
  std::ifstream file;
  file.open(filepath.c_str());

  int header_count = 0;
  if (file.is_open()){
    while (getline(file,line)){ //check how many "_reflns.B_iso_Wilson_estimate"
      header = getHeader(line, 29);
      if (header == "_reflns.B_iso_Wilson_estimate"){
	header_count += 1;
      }
    }
      
    if (header_count >= 2){//if there are 2+, remove 1st
      bool tf_remove = true; //set bool var as a switch for one-time use
      file.clear(); //rewind file pointer
      file.seekg(0);
	
      std::string filepath_out = "temp_reflns.cif";
      std::ofstream file_out;
      file_out.open(filepath_out.c_str());
	  
      while (getline(file,line)){
	header = getHeader(line, 29);
	if (header == "_reflns.B_iso_Wilson_estimate" && tf_remove){
	  tf_remove = false;
	}else{
	  file_out << line << '\n';
	}
      }
      file.close();
      file_out.close();

      //lines below copy the output file to original fout file name
      std::ifstream file_cleaned_from("temp_reflns.cif"); 
      std::ofstream file_cleaned_to(filepath.c_str());
      file_cleaned_to << file_cleaned_from.rdbuf();
	
    }else{
      file.close();
    }
  }else{
    std::cout << "Unable to open " << filepath << "\n";
  }
  return header_count;
}
