#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"
#include "util.h"


void cif2pdb(char *iFile, char *oFile)
// only for the purpose of getting cells and residue names
{
    char *space_group, *a, *b, *c, *alpha, *beta, *gamma;
    char **group, **natm, **symbol, **atom, **asym, **comp, **nres, **biso;
    char **occ, **x, **y, **z, **ins,**alt, **model, **aa, **bb;
    char symm[80],alter[10], inst[10], tmp1[80],atomname[10], *wave, *nfr;
    int i=0,  j=0;
    long n0=0,n1=0,n2=0,n3=0,n4=0,n5=0,n6=0;
    long n7=0,n8=0,n9=0, n10=0, n11=0, n12=0, nn=0;

    string diags, blockId;
    CifFile * fobjR = NULL;
    FILE *fp, *fw;
    
    if((fp = fopen(iFile, "r"))==NULL){        
        printf("Can not open the file %s (cif2pdb)\n",iFile);
        return ;
    }
    if((fw = fopen(oFile, "w"))==NULL){        
        printf("Can not write the file %s (cif2pdb)\n",oFile);
        return ;
    }

  
    CifParser* cifParserR = NULL;
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(iFile, diags);
    delete(cifParserR);

    if (!diags.empty()) {
        fprintf(stderr, "Diags for file %s: %s\n",iFile,diags.c_str());
//        exit(0);
    }
    
    blockId = fobjR->GetFirstBlockName();

    group=get_attribute_values(fobjR,blockId.c_str(),"atom_site", "group_PDB", &n1);
    natm=get_attribute_values(fobjR,blockId.c_str(),"atom_site", "id", &n2);
    atom = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "auth_atom_id", &n3);
    alt = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "label_alt_id", &n4);
    comp = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "auth_comp_id", &n5);
    asym = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "auth_asym_id", &n6);
    nres = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "auth_seq_id", &n7);
    ins = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "pdbx_PDB_ins_code", &n7);
    if (!ins) ins = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "ndb_ins_code", &n7);
    x = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "Cartn_x", &n8  );
    y = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "Cartn_y", &n9  );
    z = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "Cartn_z", &n10 );
    occ = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "occupancy", &n11 );
    biso = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "B_iso_or_equiv", &n12 );
    model = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "pdbx_PDB_model_num", &n0);
    symbol = get_attribute_values(fobjR, blockId.c_str(), "atom_site", "type_symbol", &n0);
    

    a = get_attribute_value(fobjR, blockId.c_str(), "database", "ndb_code_PDB");
    if (a){
        fprintf(fw,"HEADER%60s\n",  a);
    }else{
        aa = get_attribute_values(fobjR, blockId.c_str(), "database_2", "database_id", &nn);
        bb = get_attribute_values(fobjR, blockId.c_str(), "database_2", "database_code", &nn);
        for(i=0; i<nn; i++){
            if (aa && !strcmp(aa[i], "PDB")){
                fprintf(fw,"HEADER%60s\n",  bb[i]);
                break;
            }
        }
    }
    
    nfr=get_attribute_value(fobjR, blockId.c_str(), "refine", "ls_d_res_high");
    if (nfr){
        fprintf(fw,"REMARK   3   RESOLUTION RANGE HIGH (ANGSTROMS) : %s\n",nfr);
    }

    nfr=get_attribute_value(fobjR, blockId.c_str(), "refine", "ls_d_res_low");
    if (nfr){
        fprintf(fw,"REMARK   3   RESOLUTION RANGE LOW  (ANGSTROMS) : %s\n",nfr);
    }

     
    nfr=get_attribute_value(fobjR, blockId.c_str(), "refine", "ls_number_reflns_R_free");
    if (nfr){
        fprintf(fw,"REMARK   3   FREE R VALUE TEST SET COUNT      : %s\n",nfr);
    }

    wave=get_attribute_value(fobjR, blockId.c_str(), "diffrn_radiation_wavelength", "wavelength");
    if(wave){
        fprintf(fw,"REMARK 200  WAVELENGTH OR RANGE        (A) : %s\n", wave);
    }else{ //try the order.cif
        wave=get_attribute_value(fobjR, blockId.c_str(), "diffrn_source", "rcsb_wavelength_list");
        if (wave){
            fprintf(fw,"REMARK 200  WAVELENGTH OR RANGE        (A) : %s\n", wave);
        }
    }
    
    a = get_attribute_value(fobjR, blockId.c_str(), "cell", "length_a");
    b = get_attribute_value(fobjR, blockId.c_str(), "cell", "length_b");
    c = get_attribute_value(fobjR, blockId.c_str(), "cell", "length_c");
    alpha = get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_alpha");
    beta = get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_beta");
    gamma = get_attribute_value(fobjR, blockId.c_str(), "cell", "angle_gamma");
    space_group = get_attribute_value(fobjR, blockId.c_str(), "symmetry", "space_group_name_H-M");
    (space_group)?strcpy(symm, space_group):strcpy(symm," ");
        

    if(a && b && c && alpha && beta && gamma){
        fprintf(fw,"CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %10s \n",
                atof(a) , atof(b) , atof(c), atof(alpha),atof(beta),atof(gamma),symm);
    }


    if (!group ||!natm ||!atom ||!alt ||!comp ||!asym || !nres||!ins ||!x 
        ||!y||!z ||!occ || !biso|| !symbol){

        printf("mmCIF to PDB conversion error! please check your input mmCIF file.\n");
        
        fclose(fp);
        fclose(fw);
//        exit(0);
        return;
    }

    
    for(i=0; i<n8; i++){
        if(model){// model id exist
            if(model[i][0] != '.' && model[i][0] != '?'){
               j=atoi(model[i]); 
               if(j!=1) continue; //only keep model 1
            }
        }  
      
        if(strlen(symbol[i])>1){ //
             sprintf(tmp1,"%s    ", atom[i]);
        }else {
             sprintf(tmp1," %s    ", atom[i]);
             if (strlen(atom[i]) == 4) sprintf(tmp1,"%s    ", atom[i]);
        }
        strncpy(atomname, tmp1, 4);
        atomname[4] = '\0';

        (strchr(ins[i],'.') || strchr(ins[i],'?'))? strcpy(inst," "): strcpy(inst,ins[i]);
        (strchr(alt[i],'.') || strchr(alt[i],'?'))? strcpy(alter," "): strcpy(alter,alt[i]);

        fprintf(fw,"%-6s",group[i]);
        fprintf(fw, "%5s %4s%1s%3s%1s%1s%4s    %8s%8s%8s%6s%6s",natm[i], atomname,
                alter,comp[i], inst,asym[i],nres[i], x[i], y[i],z[i],occ[i], biso[i]);
        fprintf(fw,"%12s  \n",symbol[i]);

  }
  fclose(fp);
  fclose(fw);
  if (fobjR) delete fobjR;
  

}




           
